@@ -146,7 +146,8 @@ void zend_exception_restore(void) /* {{{ */
 
 static zend_always_inline zend_bool is_handle_exception_set() {
 	zend_execute_data *execute_data = EG(current_execute_data);
-	return !execute_data->func
+	return !execute_data
+		|| !execute_data->func
 		|| !ZEND_USER_CODE(execute_data->func->common.type)
 		|| execute_data->opline->opcode == ZEND_HANDLE_EXCEPTION;
 }
@@ -165,6 +166,12 @@ ZEND_API ZEND_COLD void zend_throw_exception_internal(zend_object *exception) /*
 
 	if (exception != NULL) {
 		zend_object *previous = EG(exception);
+		if (previous && zend_is_unwind_exit(previous)) {
+			/* Don't replace unwinding exception with different exception. */
+			OBJ_RELEASE(exception);
+			return;
+		}
+
 		zend_exception_set_previous(exception, EG(exception));
 		EG(exception) = exception;
 		if (previous) {
