@@ -1,6 +1,4 @@
 /*
-   +----------------------------------------------------------------------+
-   | PHP Version 7                                                        |
    +----------------------------------------------------------------------+
    | Copyright (c) The PHP Group                                          |
    +----------------------------------------------------------------------+
@@ -31,6 +29,7 @@
 #include "php_com_dotnet.h"
 #include "php_com_dotnet_internal.h"
 #include "Zend/zend_exceptions.h"
+#include "com_persist_arginfo.h"
 
 /* {{{ expose php_stream as a COM IStream */
 
@@ -277,13 +276,11 @@ PHP_COM_DOTNET_API IStream *php_com_wrapper_export_stream(php_stream *stream)
 	return (IStream*)stm;
 }
 
-#define CPH_ME(fname, arginfo)	PHP_ME(com_persist, fname, arginfo, ZEND_ACC_PUBLIC)
-#define CPH_SME(fname, arginfo)	PHP_ME(com_persist, fname, arginfo, ZEND_ACC_ALLOW_STATIC|ZEND_ACC_PUBLIC)
-#define CPH_METHOD(fname)		static PHP_METHOD(com_persist, fname)
+#define CPH_METHOD(fname)		PHP_METHOD(COMPersistHelper, fname)
 
 #define CPH_FETCH()				php_com_persist_helper *helper = (php_com_persist_helper*)Z_OBJ_P(getThis());
 
-#define CPH_NO_OBJ()			if (helper->unk == NULL) { php_com_throw_exception(E_INVALIDARG, "No COM object is associated with this helper instance"); return; }
+#define CPH_NO_OBJ()			if (helper->unk == NULL) { php_com_throw_exception(E_INVALIDARG, "No COM object is associated with this helper instance"); RETURN_THROWS(); }
 
 typedef struct {
 	zend_object			std;
@@ -322,14 +319,17 @@ static inline HRESULT get_persist_file(php_com_persist_helper *helper)
 }
 
 
-/* {{{ proto string COMPersistHelper::GetCurFile()
-   Determines the filename into which an object will be saved, or false if none is set, via IPersistFile::GetCurFile */
+/* {{{ Determines the filename into which an object will be saved, or false if none is set, via IPersistFile::GetCurFile */
 CPH_METHOD(GetCurFileName)
 {
 	HRESULT res;
 	OLECHAR *olename = NULL;
 	CPH_FETCH();
 
+	if (zend_parse_parameters_none() == FAILURE) {
+		RETURN_THROWS();
+	}
+
 	CPH_NO_OBJ();
 
 	res = get_persist_file(helper);
@@ -357,8 +357,7 @@ CPH_METHOD(GetCurFileName)
 /* }}} */
 
 
-/* {{{ proto bool COMPersistHelper::SaveToFile(string filename [, bool remember])
-   Persist object data to file, via IPersistFile::Save */
+/* {{{ Persist object data to file, via IPersistFile::Save */
 CPH_METHOD(SaveToFile)
 {
 	HRESULT res;
@@ -368,16 +367,15 @@ CPH_METHOD(SaveToFile)
 	OLECHAR *olefilename = NULL;
 	CPH_FETCH();
 
+	if (FAILURE == zend_parse_parameters(ZEND_NUM_ARGS(), "p!|b",
+				&filename, &filename_len, &remember)) {
+		RETURN_THROWS();
+	}
+
 	CPH_NO_OBJ();
 
 	res = get_persist_file(helper);
 	if (helper->ipf) {
-		if (FAILURE == zend_parse_parameters(ZEND_NUM_ARGS(), "p!|b",
-					&filename, &filename_len, &remember)) {
-			php_com_throw_exception(E_INVALIDARG, "Invalid arguments");
-			return;
-		}
-
 		if (filename) {
 			fullpath = expand_filepath(filename, NULL);
 			if (!fullpath) {
@@ -420,8 +418,7 @@ CPH_METHOD(SaveToFile)
 }
 /* }}} */
 
-/* {{{ proto bool COMPersistHelper::LoadFromFile(string filename [, int flags])
-   Load object data from file, via IPersistFile::Load */
+/* {{{ Load object data from file, via IPersistFile::Load */
 CPH_METHOD(LoadFromFile)
 {
 	HRESULT res;
@@ -431,17 +428,15 @@ CPH_METHOD(LoadFromFile)
 	OLECHAR *olefilename;
 	CPH_FETCH();
 
+	if (FAILURE == zend_parse_parameters(ZEND_NUM_ARGS(), "p|l",
+				&filename, &filename_len, &flags)) {
+		RETURN_THROWS();
+	}
+
 	CPH_NO_OBJ();
 
 	res = get_persist_file(helper);
 	if (helper->ipf) {
-
-		if (FAILURE == zend_parse_parameters(ZEND_NUM_ARGS(), "p|l",
-					&filename, &filename_len, &flags)) {
-			php_com_throw_exception(E_INVALIDARG, "Invalid arguments");
-			return;
-		}
-
 		if (!(fullpath = expand_filepath(filename, NULL))) {
 			RETURN_FALSE;
 		}
@@ -467,14 +462,17 @@ CPH_METHOD(LoadFromFile)
 }
 /* }}} */
 
-/* {{{ proto int COMPersistHelper::GetMaxStreamSize()
-   Gets maximum stream size required to store the object data, via IPersistStream::GetSizeMax (or IPersistStreamInit::GetSizeMax) */
+/* {{{ Gets maximum stream size required to store the object data, via IPersistStream::GetSizeMax (or IPersistStreamInit::GetSizeMax) */
 CPH_METHOD(GetMaxStreamSize)
 {
 	HRESULT res;
 	ULARGE_INTEGER size;
 	CPH_FETCH();
 
+	if (zend_parse_parameters_none() == FAILURE) {
+		RETURN_THROWS();
+	}
+
 	CPH_NO_OBJ();
 
 	res = get_persist_stream_init(helper);
@@ -486,7 +484,7 @@ CPH_METHOD(GetMaxStreamSize)
 			res = IPersistStream_GetSizeMax(helper->ips, &size);
 		} else {
 			php_com_throw_exception(res, NULL);
-			return;
+			RETURN_THROWS();
 		}
 	}
 
@@ -499,13 +497,16 @@ CPH_METHOD(GetMaxStreamSize)
 }
 /* }}} */
 
-/* {{{ proto int COMPersistHelper::InitNew()
-   Initializes the object to a default state, via IPersistStreamInit::InitNew */
+/* {{{ Initializes the object to a default state, via IPersistStreamInit::InitNew */
 CPH_METHOD(InitNew)
 {
 	HRESULT res;
 	CPH_FETCH();
 
+	if (zend_parse_parameters_none() == FAILURE) {
+		RETURN_THROWS();
+	}
+
 	CPH_NO_OBJ();
 
 	res = get_persist_stream_init(helper);
@@ -523,8 +524,7 @@ CPH_METHOD(InitNew)
 }
 /* }}} */
 
-/* {{{ proto mixed COMPersistHelper::LoadFromStream(resource stream)
-   Initializes an object from the stream where it was previously saved, via IPersistStream::Load or OleLoadFromStream */
+/* {{{ Initializes an object from the stream where it was previously saved, via IPersistStream::Load or OleLoadFromStream */
 CPH_METHOD(LoadFromStream)
 {
 	zval *zstm;
@@ -534,21 +534,20 @@ CPH_METHOD(LoadFromStream)
 	CPH_FETCH();
 
 	if (FAILURE == zend_parse_parameters(ZEND_NUM_ARGS(), "r", &zstm)) {
-		php_com_throw_exception(E_INVALIDARG, "invalid arguments");
-		return;
+		RETURN_THROWS();
 	}
 
 	php_stream_from_zval_no_verify(stream, zstm);
 
 	if (stream == NULL) {
 		php_com_throw_exception(E_INVALIDARG, "expected a stream");
-		return;
+		RETURN_THROWS();
 	}
 
 	stm = php_com_wrapper_export_stream(stream);
 	if (stm == NULL) {
 		php_com_throw_exception(E_UNEXPECTED, "failed to wrap stream");
-		return;
+		RETURN_THROWS();
 	}
 
 	res = S_OK;
@@ -578,13 +577,12 @@ CPH_METHOD(LoadFromStream)
 
 	if (FAILED(res)) {
 		php_com_throw_exception(res, NULL);
-		RETURN_NULL();
+		RETURN_THROWS();
 	}
 }
 /* }}} */
 
-/* {{{ proto int COMPersistHelper::SaveToStream(resource stream)
-   Saves the object to a stream, via IPersistStream::Save */
+/* {{{ Saves the object to a stream, via IPersistStream::Save */
 CPH_METHOD(SaveToStream)
 {
 	zval *zstm;
@@ -596,21 +594,20 @@ CPH_METHOD(SaveToStream)
 	CPH_NO_OBJ();
 
 	if (FAILURE == zend_parse_parameters(ZEND_NUM_ARGS(), "r", &zstm)) {
-		php_com_throw_exception(E_INVALIDARG, "invalid arguments");
-		return;
+		RETURN_THROWS();
 	}
 
 	php_stream_from_zval_no_verify(stream, zstm);
 
 	if (stream == NULL) {
 		php_com_throw_exception(E_INVALIDARG, "expected a stream");
-		return;
+		RETURN_THROWS();
 	}
 
 	stm = php_com_wrapper_export_stream(stream);
 	if (stm == NULL) {
 		php_com_throw_exception(E_UNEXPECTED, "failed to wrap stream");
-		return;
+		RETURN_THROWS();
 	}
 
 	res = get_persist_stream_init(helper);
@@ -627,15 +624,14 @@ CPH_METHOD(SaveToStream)
 
 	if (FAILED(res)) {
 		php_com_throw_exception(res, NULL);
-		return;
+		RETURN_THROWS();
 	}
 
 	RETURN_TRUE;
 }
 /* }}} */
 
-/* {{{ proto COMPersistHelper::__construct([object com_object])
-   Creates a persistence helper object, usually associated with a com_object */
+/* {{{ Creates a persistence helper object, usually associated with a com_object */
 CPH_METHOD(__construct)
 {
 	php_com_dotnet_object *obj = NULL;
@@ -644,8 +640,7 @@ CPH_METHOD(__construct)
 
 	if (FAILURE == zend_parse_parameters(ZEND_NUM_ARGS(), "|O!",
 				&zobj, php_com_variant_class_entry)) {
-		php_com_throw_exception(E_INVALIDARG, "invalid arguments");
-		return;
+		RETURN_THROWS();
 	}
 
 	if (!zobj) {
@@ -656,7 +651,7 @@ CPH_METHOD(__construct)
 
 	if (V_VT(&obj->v) != VT_DISPATCH || V_DISPATCH(&obj->v) == NULL) {
 		php_com_throw_exception(E_INVALIDARG, "parameter must represent an IDispatch COM object");
-		return;
+		RETURN_THROWS();
 	}
 
 	/* it is always safe to cast an interface to IUnknown */
@@ -667,20 +662,6 @@ CPH_METHOD(__construct)
 /* }}} */
 
 
-
-
-static const zend_function_entry com_persist_helper_methods[] = {
-	CPH_ME(__construct, NULL)
-	CPH_ME(GetCurFileName, NULL)
-	CPH_ME(SaveToFile, NULL)
-	CPH_ME(LoadFromFile, NULL)
-	CPH_ME(GetMaxStreamSize, NULL)
-	CPH_ME(InitNew, NULL)
-	CPH_ME(LoadFromStream, NULL)
-	CPH_ME(SaveToStream, NULL)
-	PHP_FE_END
-};
-
 static void helper_free_storage(zend_object *obj)
 {
 	php_com_persist_helper *object = (php_com_persist_helper*)obj;
@@ -701,9 +682,9 @@ static void helper_free_storage(zend_object *obj)
 }
 
 
-static zend_object* helper_clone(zval *obj)
+static zend_object* helper_clone(zend_object *obj)
 {
-	php_com_persist_helper *clone, *object = (php_com_persist_helper*)Z_OBJ_P(obj);
+	php_com_persist_helper *clone, *object = (php_com_persist_helper*) obj;
 
 	clone = emalloc(sizeof(*object));
 	memcpy(clone, object, sizeof(*object));
@@ -746,7 +727,7 @@ int php_com_persist_minit(INIT_FUNC_ARGS)
 	helper_handlers.free_obj = helper_free_storage;
 	helper_handlers.clone_obj = helper_clone;
 
-	INIT_CLASS_ENTRY(ce, "COMPersistHelper", com_persist_helper_methods);
+	INIT_CLASS_ENTRY(ce, "COMPersistHelper", class_COMPersistHelper_methods);
 	ce.create_object = helper_new;
 	helper_ce = zend_register_internal_class(&ce);
 	helper_ce->ce_flags |= ZEND_ACC_FINAL;
