@@ -39,6 +39,12 @@
 # endif
 #endif
 
+#if (defined(__BYTE_ORDER) && defined(__BIG_ENDIAN))
+# if __BYTE_ORDER == __BIG_ENDIAN
+#  define WORDS_BIGENDIAN
+# endif
+#endif
+
 #if defined(__s390__)
 # if __BYTE_ORDER__ == __ORDER_BIG_ENDIAN__
 #  define WORDS_BIGENDIAN
@@ -168,6 +174,22 @@ static void read_32bit_header(const unsigned char **tzf, timelib_tzinfo *tz)
 	*tzf += sizeof(buffer);
 }
 
+static int detect_slim_file(timelib_tzinfo *tz)
+{
+	if (
+		(tz->_bit32.ttisgmtcnt == 0) &&
+		(tz->_bit32.ttisstdcnt == 0) &&
+		(tz->_bit32.leapcnt    == 0) &&
+		(tz->_bit32.timecnt    == 0) &&
+		(tz->_bit32.typecnt    == 1) &&
+		(tz->_bit32.charcnt    == 1)
+	) {
+		return 1;
+	}
+
+	return 0;
+}
+
 static int read_64bit_transitions(const unsigned char **tzf, timelib_tzinfo *tz)
 {
 	int64_t *buffer = NULL;
@@ -226,7 +248,7 @@ static int read_64bit_types(const unsigned char **tzf, timelib_tzinfo *tz)
 	memcpy(buffer, *tzf, sizeof(unsigned char) * 6 * tz->bit64.typecnt);
 	*tzf += sizeof(unsigned char) * 6 * tz->bit64.typecnt;
 
-	tz->type = (ttinfo*) timelib_malloc(tz->bit64.typecnt * sizeof(ttinfo));
+	tz->type = (ttinfo*) timelib_calloc(1, tz->bit64.typecnt * sizeof(ttinfo));
 	if (!tz->type) {
 		timelib_free(buffer);
 		return TIMELIB_ERROR_CANNOT_ALLOCATE;
@@ -375,6 +397,7 @@ void timelib_dump_tzinfo(timelib_tzinfo *tz)
 	printf("Geo Location:      %f,%f\n", tz->location.latitude, tz->location.longitude);
 	printf("Comments:\n%s\n",          tz->location.comments);
 	printf("BC:                %s\n",  tz->bc ? "" : "yes");
+	printf("Slim File:         %s\n",  detect_slim_file(tz) ? "yes" : "no");
 
 	printf("\n64-bit:\n");
 	printf("UTC/Local count:   " TIMELIB_ULONG_FMT "\n", (timelib_ulong) tz->bit64.ttisgmtcnt);
@@ -412,7 +435,7 @@ void timelib_dump_tzinfo(timelib_tzinfo *tz)
 	}
 }
 
-static int seek_to_tz_position(const unsigned char **tzf, char *timezone, const timelib_tzdb *tzdb)
+static int seek_to_tz_position(const unsigned char **tzf, const char *timezone, const timelib_tzdb *tzdb)
 {
 	int left = 0, right = tzdb->index_size - 1;
 
@@ -449,7 +472,7 @@ const timelib_tzdb_index_entry *timelib_timezone_identifiers_list(const timelib_
 	return tzdb->index;
 }
 
-int timelib_timezone_id_is_valid(char *timezone, const timelib_tzdb *tzdb)
+int timelib_timezone_id_is_valid(const char *timezone, const timelib_tzdb *tzdb)
 {
 	const unsigned char *tzf;
 	return (seek_to_tz_position(&tzf, timezone, tzdb));
@@ -482,7 +505,7 @@ static void read_64bit_header(const unsigned char **tzf, timelib_tzinfo *tz)
 	*tzf += sizeof(buffer);
 }
 
-static timelib_tzinfo* timelib_tzinfo_ctor(char *name)
+static timelib_tzinfo* timelib_tzinfo_ctor(const char *name)
 {
 	timelib_tzinfo *t;
 	t = timelib_calloc(1, sizeof(timelib_tzinfo));
@@ -491,7 +514,7 @@ static timelib_tzinfo* timelib_tzinfo_ctor(char *name)
 	return t;
 }
 
-timelib_tzinfo *timelib_parse_tzfile(char *timezone, const timelib_tzdb *tzdb, int *error_code)
+timelib_tzinfo *timelib_parse_tzfile(const char *timezone, const timelib_tzdb *tzdb, int *error_code)
 {
 	const unsigned char *tzf;
 	timelib_tzinfo *tmp;
@@ -499,6 +522,8 @@ timelib_tzinfo *timelib_parse_tzfile(char *timezone, const timelib_tzdb *tzdb, i
 	int transitions_result, types_result;
 	unsigned int type; /* TIMELIB_TZINFO_PHP or TIMELIB_TZINFO_ZONEINFO */
 
+	*error_code = TIMELIB_ERROR_NO_ERROR;
+
 	if (seek_to_tz_position(&tzf, timezone, tzdb)) {
 		tmp = timelib_tzinfo_ctor(timezone);
 
@@ -514,6 +539,10 @@ timelib_tzinfo *timelib_parse_tzfile(char *timezone, const timelib_tzdb *tzdb, i
 		skip_32bit_transitions(&tzf, tmp);
 		skip_32bit_types(&tzf, tmp);
 
+		if (detect_slim_file(tmp)) {
+			*error_code = TIMELIB_ERROR_SLIM_FILE;
+		}
+
 		if (!skip_64bit_preamble(&tzf, tmp)) {
 			/* 64 bit preamble is not in place */
 			*error_code = TIMELIB_ERROR_CORRUPT_NO_64BIT_PREAMBLE;
