@@ -1,6 +1,4 @@
 /*
-   +----------------------------------------------------------------------+
-   | PHP Version 7                                                        |
    +----------------------------------------------------------------------+
    | Copyright (c) The PHP Group                                          |
    +----------------------------------------------------------------------+
@@ -50,104 +48,7 @@
 #include "php_qdbm.h"
 #include "php_tcadb.h"
 #include "php_lmdb.h"
-
-/* {{{ arginfo */
-ZEND_BEGIN_ARG_INFO_EX(arginfo_dba_popen, 0, 0, 2)
-	ZEND_ARG_INFO(0, path)
-	ZEND_ARG_INFO(0, mode)
-	ZEND_ARG_INFO(0, handlername)
-	ZEND_ARG_VARIADIC_INFO(0, handler_parameters)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(arginfo_dba_open, 0, 0, 2)
-	ZEND_ARG_INFO(0, path)
-	ZEND_ARG_INFO(0, mode)
-	ZEND_ARG_INFO(0, handlername)
-	ZEND_ARG_VARIADIC_INFO(0, handler_parameters)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO(arginfo_dba_close, 0)
-	ZEND_ARG_INFO(0, handle)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO(arginfo_dba_exists, 0)
-	ZEND_ARG_INFO(0, key)
-	ZEND_ARG_INFO(0, handle)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(arginfo_dba_fetch, 0, 0, 2)
-	ZEND_ARG_INFO(0, key)
-	ZEND_ARG_INFO(0, skip)
-	ZEND_ARG_INFO(0, handle)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO(arginfo_dba_key_split, 0)
-	ZEND_ARG_INFO(0, key)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO(arginfo_dba_firstkey, 0)
-	ZEND_ARG_INFO(0, handle)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO(arginfo_dba_nextkey, 0)
-	ZEND_ARG_INFO(0, handle)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO(arginfo_dba_delete, 0)
-	ZEND_ARG_INFO(0, key)
-	ZEND_ARG_INFO(0, handle)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO(arginfo_dba_insert, 0)
-	ZEND_ARG_INFO(0, key)
-	ZEND_ARG_INFO(0, value)
-	ZEND_ARG_INFO(0, handle)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO(arginfo_dba_replace, 0)
-	ZEND_ARG_INFO(0, key)
-	ZEND_ARG_INFO(0, value)
-	ZEND_ARG_INFO(0, handle)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO(arginfo_dba_optimize, 0)
-	ZEND_ARG_INFO(0, handle)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO(arginfo_dba_sync, 0)
-	ZEND_ARG_INFO(0, handle)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(arginfo_dba_handlers, 0, 0, 0)
-	ZEND_ARG_INFO(0, full_info)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO(arginfo_dba_list, 0)
-ZEND_END_ARG_INFO()
-
-/* }}} */
-
-/* {{{ dba_functions[]
- */
-static const zend_function_entry dba_functions[] = {
-	PHP_FE(dba_open, arginfo_dba_open)
-	PHP_FE(dba_popen, arginfo_dba_popen)
-	PHP_FE(dba_close, arginfo_dba_close)
-	PHP_FE(dba_delete, arginfo_dba_delete)
-	PHP_FE(dba_exists, arginfo_dba_exists)
-	PHP_FE(dba_fetch, arginfo_dba_fetch)
-	PHP_FE(dba_insert, arginfo_dba_insert)
-	PHP_FE(dba_replace, arginfo_dba_replace)
-	PHP_FE(dba_firstkey, arginfo_dba_firstkey)
-	PHP_FE(dba_nextkey, arginfo_dba_nextkey)
-	PHP_FE(dba_optimize, arginfo_dba_optimize)
-	PHP_FE(dba_sync, arginfo_dba_sync)
-	PHP_FE(dba_handlers, arginfo_dba_handlers)
-	PHP_FE(dba_list, arginfo_dba_list)
-	PHP_FE(dba_key_split, arginfo_dba_key_split)
-	PHP_FE_END
-};
-/* }}} */
+#include "dba_arginfo.h"
 
 PHP_MINIT_FUNCTION(dba);
 PHP_MSHUTDOWN_FUNCTION(dba);
@@ -167,7 +68,7 @@ static PHP_GINIT_FUNCTION(dba);
 zend_module_entry dba_module_entry = {
 	STANDARD_MODULE_HEADER,
 	"dba",
-	dba_functions,
+	ext_functions,
 	PHP_MINIT(dba),
 	PHP_MSHUTDOWN(dba),
 	NULL,
@@ -206,7 +107,7 @@ static size_t php_dba_make_key(zval *key, char **key_str, char **key_free)
 		size_t len;
 
 		if (zend_hash_num_elements(Z_ARRVAL_P(key)) != 2) {
-			zend_throw_error(NULL, "Key does not have exactly two elements: (key, name)");
+			zend_argument_error(NULL, 1, "must have exactly two elements: \"key\" and \"name\"");
 			return 0;
 		}
 		zend_hash_internal_pointer_reset_ex(Z_ARRVAL_P(key), &pos);
@@ -245,7 +146,7 @@ static size_t php_dba_make_key(zval *key, char **key_str, char **key_free)
 	char *key_str, *key_free;									\
 	size_t key_len; 											\
 	if (zend_parse_parameters(ac, "zr", &key, &id) == FAILURE) { 	\
-		return; 												\
+		RETURN_THROWS();										\
 	} 															\
 	if ((key_len = php_dba_make_key(key, &key_str, &key_free)) == 0) {\
 		RETURN_FALSE;											\
@@ -259,12 +160,12 @@ static size_t php_dba_make_key(zval *key, char **key_str, char **key_free)
 	switch(ac) {												\
 	case 2: 													\
 		if (zend_parse_parameters(ac, "zr", &key, &id) == FAILURE) { \
-			return;												\
+			RETURN_THROWS();									\
 		} 														\
 		break;  												\
 	case 3: 													\
 		if (zend_parse_parameters(ac, "zlr", &key, &skip, &id) == FAILURE) { \
-			return;												\
+			RETURN_THROWS();									\
 		} 														\
 		break;  												\
 	default:													\
@@ -277,13 +178,13 @@ static size_t php_dba_make_key(zval *key, char **key_str, char **key_free)
 
 #define DBA_FETCH_RESOURCE(info, id)	\
 	if ((info = (dba_info *)zend_fetch_resource2(Z_RES_P(id), "DBA identifier", le_db, le_pdb)) == NULL) { \
-		RETURN_FALSE; \
+		RETURN_THROWS(); \
 	}
 
 #define DBA_FETCH_RESOURCE_WITH_ID(info, id)	\
 	if ((info = (dba_info *)zend_fetch_resource2(Z_RES_P(id), "DBA identifier", le_db, le_pdb)) == NULL) { \
 		DBA_ID_DONE; \
-		RETURN_FALSE; \
+		RETURN_THROWS(); \
 	}
 
 #define DBA_ID_GET2   DBA_ID_PARS; DBA_GET2;   DBA_FETCH_RESOURCE_WITH_ID(info, id)
@@ -418,8 +319,7 @@ PHPAPI dba_handler *dba_get_handler(const char* handler_name)
 */
 /* }}} */
 
-/* {{{ dba_close
- */
+/* {{{ dba_close */
 static void dba_close(dba_info *info)
 {
 	if (info->hnd) {
@@ -449,8 +349,7 @@ static void dba_close(dba_info *info)
 }
 /* }}} */
 
-/* {{{ dba_close_rsrc
- */
+/* {{{ dba_close_rsrc */
 static void dba_close_rsrc(zend_resource *rsrc)
 {
 	dba_info *info = (dba_info *)rsrc->ptr;
@@ -484,8 +383,7 @@ static void dba_close_pe_rsrc(zend_resource *rsrc)
 }
 /* }}} */
 
-/* {{{ PHP_INI
- */
+/* {{{ PHP_INI */
 ZEND_INI_MH(OnUpdateDefaultHandler)
 {
 	dba_handler *hptr;
@@ -510,8 +408,7 @@ PHP_INI_BEGIN()
 PHP_INI_END()
 /* }}} */
 
-/* {{{ PHP_GINIT_FUNCTION
- */
+/* {{{ PHP_GINIT_FUNCTION */
 static PHP_GINIT_FUNCTION(dba)
 {
 #if defined(COMPILE_DL_DBA) && defined(ZTS)
@@ -522,8 +419,7 @@ static PHP_GINIT_FUNCTION(dba)
 }
 /* }}} */
 
-/* {{{ PHP_MINIT_FUNCTION
- */
+/* {{{ PHP_MINIT_FUNCTION */
 PHP_MINIT_FUNCTION(dba)
 {
 	REGISTER_INI_ENTRIES();
@@ -533,8 +429,7 @@ PHP_MINIT_FUNCTION(dba)
 }
 /* }}} */
 
-/* {{{ PHP_MSHUTDOWN_FUNCTION
- */
+/* {{{ PHP_MSHUTDOWN_FUNCTION */
 PHP_MSHUTDOWN_FUNCTION(dba)
 {
 	UNREGISTER_INI_ENTRIES();
@@ -544,8 +439,7 @@ PHP_MSHUTDOWN_FUNCTION(dba)
 
 #include "zend_smart_str.h"
 
-/* {{{ PHP_MINFO_FUNCTION
- */
+/* {{{ PHP_MINFO_FUNCTION */
 PHP_MINFO_FUNCTION(dba)
 {
 	dba_handler *hptr;
@@ -570,8 +464,7 @@ PHP_MINFO_FUNCTION(dba)
 }
 /* }}} */
 
-/* {{{ php_dba_update
- */
+/* {{{ php_dba_update */
 static void php_dba_update(INTERNAL_FUNCTION_PARAMETERS, int mode)
 {
 	size_t val_len;
@@ -584,7 +477,7 @@ static void php_dba_update(INTERNAL_FUNCTION_PARAMETERS, int mode)
 	size_t key_len;
 
 	if (zend_parse_parameters(ac, "zsr", &key, &val, &val_len, &id) == FAILURE) {
-		return;
+		RETURN_THROWS();
 	}
 
 	if ((key_len = php_dba_make_key(key, &key_str, &key_free)) == 0) {
@@ -607,8 +500,7 @@ static void php_dba_update(INTERNAL_FUNCTION_PARAMETERS, int mode)
 
 #define FREENOW if(args) {int i; for (i=0; i<ac; i++) { zval_ptr_dtor(&args[i]); } efree(args);} if(key) efree(key)
 
-/* {{{ php_find_dbm
- */
+/* {{{ php_find_dbm */
 dba_info *php_dba_find(const char* path)
 {
 	zend_resource *le;
@@ -632,8 +524,7 @@ dba_info *php_dba_find(const char* path)
 }
 /* }}} */
 
-/* {{{ php_dba_open
- */
+/* {{{ php_dba_open */
 static void php_dba_open(INTERNAL_FUNCTION_PARAMETERS, int persistent)
 {
 	zval *args = NULL;
@@ -675,7 +566,7 @@ static void php_dba_open(INTERNAL_FUNCTION_PARAMETERS, int persistent)
 	/* Exception during string conversion */
 	if (EG(exception)) {
 		FREENOW;
-		return;
+		RETURN_THROWS();
 	}
 
 	if (persistent) {
@@ -969,11 +860,10 @@ restart:
 				fcntl(info->fd, F_SETFL, flags & ~O_APPEND);
 #elif defined(PHP_WIN32)
 			} else if (modenr == DBA_CREAT && need_creation && !restarted) {
-				zend_bool close_both;
-
-				close_both = (info->fp != info->lock.fp);
-				php_stream_free(info->lock.fp, persistent ? PHP_STREAM_FREE_CLOSE_PERSISTENT : PHP_STREAM_FREE_CLOSE);
-				if (close_both) {
+				if (info->lock.fp != NULL) {
+					php_stream_free(info->lock.fp, persistent ? PHP_STREAM_FREE_CLOSE_PERSISTENT : PHP_STREAM_FREE_CLOSE);
+				}
+				if (info->fp != info->lock.fp) {
 					php_stream_free(info->fp, persistent ? PHP_STREAM_FREE_CLOSE_PERSISTENT : PHP_STREAM_FREE_CLOSE);
 				}
 				info->fp = NULL;
@@ -1017,31 +907,28 @@ restart:
 /* }}} */
 #undef FREENOW
 
-/* {{{ proto resource dba_popen(string path, string mode [, string handlername, string ...])
-   Opens path using the specified handler in mode persistently */
+/* {{{ Opens path using the specified handler in mode persistently */
 PHP_FUNCTION(dba_popen)
 {
 	php_dba_open(INTERNAL_FUNCTION_PARAM_PASSTHRU, 1);
 }
 /* }}} */
 
-/* {{{ proto resource dba_open(string path, string mode [, string handlername, string ...])
-   Opens path using the specified handler in mode*/
+/* {{{ Opens path using the specified handler in mode*/
 PHP_FUNCTION(dba_open)
 {
 	php_dba_open(INTERNAL_FUNCTION_PARAM_PASSTHRU, 0);
 }
 /* }}} */
 
-/* {{{ proto void dba_close(resource handle)
-   Closes database */
+/* {{{ Closes database */
 PHP_FUNCTION(dba_close)
 {
 	zval *id;
 	dba_info *info = NULL;
 
 	if (zend_parse_parameters(ZEND_NUM_ARGS(), "r", &id) == FAILURE) {
-		return;
+		RETURN_THROWS();
 	}
 
 	DBA_FETCH_RESOURCE(info, id);
@@ -1050,8 +937,7 @@ PHP_FUNCTION(dba_close)
 }
 /* }}} */
 
-/* {{{ proto bool dba_exists(string key, resource handle)
-   Checks, if the specified key exists */
+/* {{{ Checks, if the specified key exists */
 PHP_FUNCTION(dba_exists)
 {
 	DBA_ID_GET2;
@@ -1065,8 +951,7 @@ PHP_FUNCTION(dba_exists)
 }
 /* }}} */
 
-/* {{{ proto string dba_fetch(string key, [int skip ,] resource handle)
-   Fetches the data associated with key */
+/* {{{ Fetches the data associated with key */
 PHP_FUNCTION(dba_fetch)
 {
 	char *val;
@@ -1080,8 +965,8 @@ PHP_FUNCTION(dba_fetch)
 				skip = 0;
 			}
 		} else if (!strcmp(info->hnd->name, "inifile")) {
-			/* "-1" is compareable to 0 but allows a non restrictive
-			 * access which is fater. For example 'inifile' uses this
+			/* "-1" is comparable to 0 but allows a non restrictive
+			 * access which is faster. For example 'inifile' uses this
 			 * to allow faster access when the key was already found
 			 * using firstkey/nextkey. However explicitly setting the
 			 * value to 0 ensures the first value.
@@ -1108,8 +993,7 @@ PHP_FUNCTION(dba_fetch)
 }
 /* }}} */
 
-/* {{{ proto array|false dba_key_split(string key)
-   Splits an inifile key into an array of the form array(0=>group,1=>value_name) but returns false if input is false or null */
+/* {{{ Splits an inifile key into an array of the form array(0=>group,1=>value_name) but returns false if input is false or null */
 PHP_FUNCTION(dba_key_split)
 {
 	zval *zkey;
@@ -1125,7 +1009,7 @@ PHP_FUNCTION(dba_key_split)
 		}
 	}
 	if (zend_parse_parameters(ZEND_NUM_ARGS(), "s", &key, &key_len) == FAILURE) {
-		RETURN_BOOL(0);
+		RETURN_THROWS();
 	}
 	array_init(return_value);
 	if (key[0] == '[' && (name = strchr(key, ']')) != NULL) {
@@ -1138,8 +1022,7 @@ PHP_FUNCTION(dba_key_split)
 }
 /* }}} */
 
-/* {{{ proto string dba_firstkey(resource handle)
-   Resets the internal key pointer and returns the first key */
+/* {{{ Resets the internal key pointer and returns the first key */
 PHP_FUNCTION(dba_firstkey)
 {
 	char *fkey;
@@ -1148,7 +1031,7 @@ PHP_FUNCTION(dba_firstkey)
 	dba_info *info = NULL;
 
 	if (zend_parse_parameters(ZEND_NUM_ARGS(), "r", &id) == FAILURE) {
-		return;
+		RETURN_THROWS();
 	}
 
 	DBA_FETCH_RESOURCE(info, id);
@@ -1165,8 +1048,7 @@ PHP_FUNCTION(dba_firstkey)
 }
 /* }}} */
 
-/* {{{ proto string dba_nextkey(resource handle)
-   Returns the next key */
+/* {{{ Returns the next key */
 PHP_FUNCTION(dba_nextkey)
 {
 	char *nkey;
@@ -1175,7 +1057,7 @@ PHP_FUNCTION(dba_nextkey)
 	dba_info *info = NULL;
 
 	if (zend_parse_parameters(ZEND_NUM_ARGS(), "r", &id) == FAILURE) {
-		return;
+		RETURN_THROWS();
 	}
 
 	DBA_FETCH_RESOURCE(info, id);
@@ -1192,8 +1074,7 @@ PHP_FUNCTION(dba_nextkey)
 }
 /* }}} */
 
-/* {{{ proto bool dba_delete(string key, resource handle)
-   Deletes the entry associated with key
+/* {{{ Deletes the entry associated with key
    If inifile: remove all other key lines */
 PHP_FUNCTION(dba_delete)
 {
@@ -1211,8 +1092,7 @@ PHP_FUNCTION(dba_delete)
 }
 /* }}} */
 
-/* {{{ proto bool dba_insert(string key, string value, resource handle)
-   If not inifile: Insert value as key, return false, if key exists already
+/* {{{ If not inifile: Insert value as key, return false, if key exists already
    If inifile: Add vakue as key (next instance of key) */
 PHP_FUNCTION(dba_insert)
 {
@@ -1220,8 +1100,7 @@ PHP_FUNCTION(dba_insert)
 }
 /* }}} */
 
-/* {{{ proto bool dba_replace(string key, string value, resource handle)
-   Inserts value as key, replaces key, if key exists already
+/* {{{ Inserts value as key, replaces key, if key exists already
    If inifile: remove all other key lines */
 PHP_FUNCTION(dba_replace)
 {
@@ -1229,15 +1108,14 @@ PHP_FUNCTION(dba_replace)
 }
 /* }}} */
 
-/* {{{ proto bool dba_optimize(resource handle)
-   Optimizes (e.g. clean up, vacuum) database */
+/* {{{ Optimizes (e.g. clean up, vacuum) database */
 PHP_FUNCTION(dba_optimize)
 {
 	zval *id;
 	dba_info *info = NULL;
 
 	if (zend_parse_parameters(ZEND_NUM_ARGS(), "r", &id) == FAILURE) {
-		return;
+		RETURN_THROWS();
 	}
 
 	DBA_FETCH_RESOURCE(info, id);
@@ -1252,15 +1130,14 @@ PHP_FUNCTION(dba_optimize)
 }
 /* }}} */
 
-/* {{{ proto bool dba_sync(resource handle)
-   Synchronizes database */
+/* {{{ Synchronizes database */
 PHP_FUNCTION(dba_sync)
 {
 	zval *id;
 	dba_info *info = NULL;
 
 	if (zend_parse_parameters(ZEND_NUM_ARGS(), "r", &id) == FAILURE) {
-		return;
+		RETURN_THROWS();
 	}
 
 	DBA_FETCH_RESOURCE(info, id);
@@ -1273,15 +1150,14 @@ PHP_FUNCTION(dba_sync)
 }
 /* }}} */
 
-/* {{{ proto array dba_handlers([bool full_info])
-   List configured database handlers */
+/* {{{ List configured database handlers */
 PHP_FUNCTION(dba_handlers)
 {
 	dba_handler *hptr;
 	zend_bool full_info = 0;
 
 	if (zend_parse_parameters(ZEND_NUM_ARGS(), "|b", &full_info) == FAILURE) {
-		RETURN_FALSE;
+		RETURN_THROWS();
 	}
 
 	array_init(return_value);
@@ -1299,8 +1175,7 @@ PHP_FUNCTION(dba_handlers)
 }
 /* }}} */
 
-/* {{{ proto array dba_list()
-   List opened databases */
+/* {{{ List opened databases */
 PHP_FUNCTION(dba_list)
 {
 	zend_ulong numitems, i;
@@ -1308,7 +1183,7 @@ PHP_FUNCTION(dba_list)
 	dba_info *info;
 
 	if (zend_parse_parameters_none() == FAILURE) {
-		RETURN_FALSE;
+		RETURN_THROWS();
 	}
 
 	array_init(return_value);
