@@ -1,6 +1,4 @@
 /*
-  +----------------------------------------------------------------------+
-  | PHP Version 7                                                        |
   +----------------------------------------------------------------------+
   | Copyright (c) The PHP Group                                          |
   +----------------------------------------------------------------------+
@@ -83,11 +81,8 @@ PHP_DOM_EXPORT xmlNodePtr dom_object_get_node(dom_object *obj);
 #define DOM_RET_OBJ(obj, ret, domobject) \
 	*ret = php_dom_create_object(obj, return_value, domobject)
 
-#define DOM_GET_THIS(zval) \
-	do { zval = ZEND_THIS; } while (0)
-
 #define DOM_GET_THIS_OBJ(__ptr, __id, __prtype, __intern) \
-	DOM_GET_THIS(__id); \
+	__id = ZEND_THIS; \
 	DOM_GET_OBJ(__ptr, __id, __prtype, __intern);
 
 #endif
