@@ -1,8 +1,6 @@
 #!/usr/bin/env php
 <?php
 /*
-   +----------------------------------------------------------------------+
-   | PHP Version 7                                                        |
    +----------------------------------------------------------------------+
    | Copyright (c) The PHP Group                                          |
    +----------------------------------------------------------------------+
@@ -18,18 +16,16 @@
    +----------------------------------------------------------------------+
 */
 
-/* $Id$ */
+/* $Id: 8a3cf76e293233efd7dfd68549c0f816008ec254 $ */
 
-/* {{{ error
- */
+/* {{{ error */
 function error($message) {
     printf('Error: %s%s', $message, PHP_EOL);
     exit;
 }
 /* }}} */
 
-/* {{{ print_help
- */
+/* {{{ print_help */
 function print_help() {
     if (PHP_OS_FAMILY != 'Windows') {
         $file_prefix = './';
@@ -120,8 +116,7 @@ HELP;
 }
 /* }}} */
 
-/* {{{ task
- */
+/* {{{ task */
 function task($label, $callback) {
     printf('%s... ', $label);
 
@@ -131,8 +126,7 @@ function task($label, $callback) {
 }
 /* }}} */
 
-/* {{{ print_success
- */
+/* {{{ print_success */
 function print_success() {
     global $options;
 
@@ -146,7 +140,7 @@ function print_success() {
 
     printf('%1$sSuccess. The extension is now ready to be compiled. To do so, use the%s', PHP_EOL);
     printf('following steps:%1$s%1$s', PHP_EOL);
-    printf('cd /path/to/php-src/%s%s', $options['ext'], PHP_EOL);
+    printf('cd /path/to/php-src/ext/%s%s', $options['ext'], PHP_EOL);
     printf('phpize%s', PHP_EOL);
     printf('%sconfigure%s', $file_prefix, PHP_EOL);
     printf('%smake%2$s%2$s', $make_prefix, PHP_EOL);
@@ -156,8 +150,7 @@ function print_success() {
 }
 /* }}} */
 
-/* {{{ process_args
- */
+/* {{{ process_args */
 function process_args($argv, $argc) {
     $options = [
             'unix'		=> true,
@@ -239,8 +232,7 @@ function process_args($argv, $argc) {
 }
 /* }}} */
 
-/* {{{ process_source_tags
- */
+/* {{{ process_source_tags */
 function process_source_tags($file, $short_name) {
     global $options;
 
@@ -263,8 +255,6 @@ function process_source_tags($file, $short_name) {
 
                 $header = <<<"HEADER"
 /*
-   +----------------------------------------------------------------------+
-   | PHP Version 7                                                        |
    +----------------------------------------------------------------------+
    | Copyright (c) The PHP Group                                          |
    +----------------------------------------------------------------------+
@@ -298,8 +288,7 @@ HEADER;
 }
 /* }}} */
 
-/* {{{ copy_config_scripts
- */
+/* {{{ copy_config_scripts */
 function copy_config_scripts() {
     global $options;
 
@@ -327,14 +316,15 @@ function copy_config_scripts() {
 }
 /* }}} */
 
-/* {{{ copy_sources
- */
+/* {{{ copy_sources */
 function copy_sources() {
     global $options;
 
     $files = [
             'skeleton.c'		=> $options['ext'] . '.c',
-            'php_skeleton.h'	=> 'php_' . $options['ext'] . '.h'
+            'skeleton.stub.php'	=> $options['ext'] . '.stub.php',
+            'php_skeleton.h'	=> 'php_' . $options['ext'] . '.h',
+            'skeleton_arginfo.h' => $options['ext'] . '_arginfo.h'
             ];
 
     foreach ($files as $src_file => $dst_file) {
@@ -347,8 +337,7 @@ function copy_sources() {
 }
 /* }}} */
 
-/* {{{ copy_tests
- */
+/* {{{ copy_tests */
 function copy_tests() {
     global $options;
 
