@@ -1,6 +1,4 @@
 /*
-   +----------------------------------------------------------------------+
-   | PHP Version 7                                                        |
    +----------------------------------------------------------------------+
    | Copyright (c) The PHP Group                                          |
    +----------------------------------------------------------------------+
@@ -23,8 +21,6 @@
 
 #include "php.h"
 
-#if HAVE_FTP
-
 #include <stdio.h>
 #include <ctype.h>
 #include <stdlib.h>
@@ -47,7 +43,7 @@
 #endif
 #include <errno.h>
 
-#if HAVE_SYS_TIME_H
+#ifdef HAVE_SYS_TIME_H
 #include <sys/time.h>
 #endif
 
@@ -116,8 +112,7 @@ union ipbox {
 	unsigned char	c[8];
 };
 
-/* {{{ ftp_open
- */
+/* {{{ ftp_open */
 ftpbuf_t*
 ftp_open(const char *host, short port, zend_long timeout_sec)
 {
@@ -165,8 +160,7 @@ bail:
 }
 /* }}} */
 
-/* {{{ ftp_close
- */
+/* {{{ ftp_close */
 ftpbuf_t*
 ftp_close(ftpbuf_t *ftp)
 {
@@ -193,8 +187,7 @@ ftp_close(ftpbuf_t *ftp)
 }
 /* }}} */
 
-/* {{{ ftp_gc
- */
+/* {{{ ftp_gc */
 void
 ftp_gc(ftpbuf_t *ftp)
 {
@@ -212,8 +205,7 @@ ftp_gc(ftpbuf_t *ftp)
 }
 /* }}} */
 
-/* {{{ ftp_quit
- */
+/* {{{ ftp_quit */
 int
 ftp_quit(ftpbuf_t *ftp)
 {
@@ -237,8 +229,7 @@ ftp_quit(ftpbuf_t *ftp)
 }
 /* }}} */
 
-/* {{{ ftp_login
- */
+/* {{{ ftp_login */
 int
 ftp_login(ftpbuf_t *ftp, const char *user, const size_t user_len, const char *pass, const size_t pass_len)
 {
@@ -279,7 +270,7 @@ ftp_login(ftpbuf_t *ftp, const char *user, const size_t user_len, const char *pa
 
 		ctx = SSL_CTX_new(SSLv23_client_method());
 		if (ctx == NULL) {
-			php_error_docref(NULL, E_WARNING, "failed to create the SSL context");
+			php_error_docref(NULL, E_WARNING, "Failed to create the SSL context");
 			return 0;
 		}
 
@@ -295,7 +286,7 @@ ftp_login(ftpbuf_t *ftp, const char *user, const size_t user_len, const char *pa
 		SSL_CTX_free(ctx);
 
 		if (ftp->ssl_handle == NULL) {
-			php_error_docref(NULL, E_WARNING, "failed to create the SSL handle");
+			php_error_docref(NULL, E_WARNING, "Failed to create the SSL handle");
 			return 0;
 		}
 
@@ -386,8 +377,7 @@ ftp_login(ftpbuf_t *ftp, const char *user, const size_t user_len, const char *pa
 }
 /* }}} */
 
-/* {{{ ftp_reinit
- */
+/* {{{ ftp_reinit */
 int
 ftp_reinit(ftpbuf_t *ftp)
 {
@@ -410,8 +400,7 @@ ftp_reinit(ftpbuf_t *ftp)
 }
 /* }}} */
 
-/* {{{ ftp_syst
- */
+/* {{{ ftp_syst */
 const char*
 ftp_syst(ftpbuf_t *ftp)
 {
@@ -446,8 +435,7 @@ ftp_syst(ftpbuf_t *ftp)
 }
 /* }}} */
 
-/* {{{ ftp_pwd
- */
+/* {{{ ftp_pwd */
 const char*
 ftp_pwd(ftpbuf_t *ftp)
 {
@@ -480,8 +468,7 @@ ftp_pwd(ftpbuf_t *ftp)
 }
 /* }}} */
 
-/* {{{ ftp_exec
- */
+/* {{{ ftp_exec */
 int
 ftp_exec(ftpbuf_t *ftp, const char *cmd, const size_t cmd_len)
 {
@@ -499,8 +486,7 @@ ftp_exec(ftpbuf_t *ftp, const char *cmd, const size_t cmd_len)
 }
 /* }}} */
 
-/* {{{ ftp_raw
- */
+/* {{{ ftp_raw */
 void
 ftp_raw(ftpbuf_t *ftp, const char *cmd, const size_t cmd_len, zval *return_value)
 {
@@ -520,8 +506,7 @@ ftp_raw(ftpbuf_t *ftp, const char *cmd, const size_t cmd_len, zval *return_value
 }
 /* }}} */
 
-/* {{{ ftp_chdir
- */
+/* {{{ ftp_chdir */
 int
 ftp_chdir(ftpbuf_t *ftp, const char *dir, const size_t dir_len)
 {
@@ -544,8 +529,7 @@ ftp_chdir(ftpbuf_t *ftp, const char *dir, const size_t dir_len)
 }
 /* }}} */
 
-/* {{{ ftp_cdup
- */
+/* {{{ ftp_cdup */
 int
 ftp_cdup(ftpbuf_t *ftp)
 {
@@ -568,8 +552,7 @@ ftp_cdup(ftpbuf_t *ftp)
 }
 /* }}} */
 
-/* {{{ ftp_mkdir
- */
+/* {{{ ftp_mkdir */
 zend_string*
 ftp_mkdir(ftpbuf_t *ftp, const char *dir, const size_t dir_len)
 {
@@ -600,8 +583,7 @@ ftp_mkdir(ftpbuf_t *ftp, const char *dir, const size_t dir_len)
 }
 /* }}} */
 
-/* {{{ ftp_rmdir
- */
+/* {{{ ftp_rmdir */
 int
 ftp_rmdir(ftpbuf_t *ftp, const char *dir, const size_t dir_len)
 {
@@ -618,8 +600,7 @@ ftp_rmdir(ftpbuf_t *ftp, const char *dir, const size_t dir_len)
 }
 /* }}} */
 
-/* {{{ ftp_chmod
- */
+/* {{{ ftp_chmod */
 int
 ftp_chmod(ftpbuf_t *ftp, const int mode, const char *filename, const int filename_len)
 {
@@ -651,8 +632,7 @@ ftp_chmod(ftpbuf_t *ftp, const int mode, const char *filename, const int filenam
 }
 /* }}} */
 
-/* {{{ ftp_alloc
- */
+/* {{{ ftp_alloc */
 int
 ftp_alloc(ftpbuf_t *ftp, const zend_long size, zend_string **response)
 {
@@ -689,8 +669,7 @@ ftp_alloc(ftpbuf_t *ftp, const zend_long size, zend_string **response)
 }
 /* }}} */
 
-/* {{{ ftp_nlist
- */
+/* {{{ ftp_nlist */
 char**
 ftp_nlist(ftpbuf_t *ftp, const char *path, const size_t path_len)
 {
@@ -698,8 +677,7 @@ ftp_nlist(ftpbuf_t *ftp, const char *path, const size_t path_len)
 }
 /* }}} */
 
-/* {{{ ftp_list
- */
+/* {{{ ftp_list */
 char**
 ftp_list(ftpbuf_t *ftp, const char *path, const size_t path_len, int recursive)
 {
@@ -707,8 +685,7 @@ ftp_list(ftpbuf_t *ftp, const char *path, const size_t path_len, int recursive)
 }
 /* }}} */
 
-/* {{{ ftp_mlsd
- */
+/* {{{ ftp_mlsd */
 char**
 ftp_mlsd(ftpbuf_t *ftp, const char *path, const size_t path_len)
 {
@@ -716,8 +693,7 @@ ftp_mlsd(ftpbuf_t *ftp, const char *path, const size_t path_len)
 }
 /* }}} */
 
-/* {{{ ftp_mlsd_parse_line
- */
+/* {{{ ftp_mlsd_parse_line */
 int
 ftp_mlsd_parse_line(HashTable *ht, const char *input) {
 
@@ -761,8 +737,7 @@ ftp_mlsd_parse_line(HashTable *ht, const char *input) {
 }
 /* }}} */
 
-/* {{{ ftp_type
- */
+/* {{{ ftp_type */
 int
 ftp_type(ftpbuf_t *ftp, ftptype_t type)
 {
@@ -793,8 +768,7 @@ ftp_type(ftpbuf_t *ftp, ftptype_t type)
 }
 /* }}} */
 
-/* {{{ ftp_pasv
- */
+/* {{{ ftp_pasv */
 int
 ftp_pasv(ftpbuf_t *ftp, int pasv)
 {
@@ -823,7 +797,7 @@ ftp_pasv(ftpbuf_t *ftp, int pasv)
 		return 0;
 	}
 
-#if HAVE_IPV6
+#ifdef HAVE_IPV6
 	if (sa->sa_family == AF_INET6) {
 		struct sockaddr_in6 *sin6 = (struct sockaddr_in6 *) sa;
 		char *endptr, delimiter;
@@ -887,8 +861,7 @@ ftp_pasv(ftpbuf_t *ftp, int pasv)
 }
 /* }}} */
 
-/* {{{ ftp_get
- */
+/* {{{ ftp_get */
 int
 ftp_get(ftpbuf_t *ftp, php_stream *outstream, const char *path, const size_t path_len, ftptype_t type, zend_long resumepos)
 {
@@ -983,8 +956,7 @@ bail:
 }
 /* }}} */
 
-/* {{{ ftp_put
- */
+/* {{{ ftp_put */
 int
 ftp_put(ftpbuf_t *ftp, const char *path, const size_t path_len, php_stream *instream, ftptype_t type, zend_long startpos)
 {
@@ -1066,8 +1038,7 @@ bail:
 /* }}} */
 
 
-/* {{{ ftp_append
- */
+/* {{{ ftp_append */
 int
 ftp_append(ftpbuf_t *ftp, const char *path, const size_t path_len, php_stream *instream, ftptype_t type)
 {
@@ -1133,8 +1104,7 @@ bail:
 }
 /* }}} */
 
-/* {{{ ftp_size
- */
+/* {{{ ftp_size */
 zend_long
 ftp_size(ftpbuf_t *ftp, const char *path, const size_t path_len)
 {
@@ -1157,8 +1127,7 @@ ftp_size(ftpbuf_t *ftp, const char *path, const size_t path_len)
 }
 /* }}} */
 
-/* {{{ ftp_mdtm
- */
+/* {{{ ftp_mdtm */
 time_t
 ftp_mdtm(ftpbuf_t *ftp, const char *path, const size_t path_len)
 {
@@ -1205,8 +1174,7 @@ ftp_mdtm(ftpbuf_t *ftp, const char *path, const size_t path_len)
 }
 /* }}} */
 
-/* {{{ ftp_delete
- */
+/* {{{ ftp_delete */
 int
 ftp_delete(ftpbuf_t *ftp, const char *path, const size_t path_len)
 {
@@ -1224,8 +1192,7 @@ ftp_delete(ftpbuf_t *ftp, const char *path, const size_t path_len)
 }
 /* }}} */
 
-/* {{{ ftp_rename
- */
+/* {{{ ftp_rename */
 int
 ftp_rename(ftpbuf_t *ftp, const char *src, const size_t src_len, const char *dest, const size_t dest_len)
 {
@@ -1248,8 +1215,7 @@ ftp_rename(ftpbuf_t *ftp, const char *src, const size_t src_len, const char *des
 }
 /* }}} */
 
-/* {{{ ftp_site
- */
+/* {{{ ftp_site */
 int
 ftp_site(ftpbuf_t *ftp, const char *cmd, const size_t cmd_len)
 {
@@ -1269,8 +1235,7 @@ ftp_site(ftpbuf_t *ftp, const char *cmd, const size_t cmd_len)
 
 /* static functions */
 
-/* {{{ ftp_putcmd
- */
+/* {{{ ftp_putcmd */
 int
 ftp_putcmd(ftpbuf_t *ftp, const char *cmd, const size_t cmd_len, const char *args, const size_t args_len)
 {
@@ -1311,8 +1276,7 @@ ftp_putcmd(ftpbuf_t *ftp, const char *cmd, const size_t cmd_len, const char *arg
 }
 /* }}} */
 
-/* {{{ ftp_readline
- */
+/* {{{ ftp_readline */
 int
 ftp_readline(ftpbuf_t *ftp)
 {
@@ -1365,8 +1329,7 @@ ftp_readline(ftpbuf_t *ftp)
 }
 /* }}} */
 
-/* {{{ ftp_getresp
- */
+/* {{{ ftp_getresp */
 int
 ftp_getresp(ftpbuf_t *ftp)
 {
@@ -1461,8 +1424,7 @@ int single_send(ftpbuf_t *ftp, php_socket_t s, void *buf, size_t size) {
 #endif
 }
 
-/* {{{ my_send
- */
+/* {{{ my_send */
 int
 my_send(ftpbuf_t *ftp, php_socket_t s, void *buf, size_t len)
 {
@@ -1499,8 +1461,7 @@ my_send(ftpbuf_t *ftp, php_socket_t s, void *buf, size_t len)
 }
 /* }}} */
 
-/* {{{ my_recv
- */
+/* {{{ my_recv */
 int
 my_recv(ftpbuf_t *ftp, php_socket_t s, void *buf, size_t len)
 {
@@ -1579,8 +1540,7 @@ my_recv(ftpbuf_t *ftp, php_socket_t s, void *buf, size_t len)
 }
 /* }}} */
 
-/* {{{ data_available
- */
+/* {{{ data_available */
 int
 data_available(ftpbuf_t *ftp, php_socket_t s)
 {
@@ -1603,8 +1563,7 @@ data_available(ftpbuf_t *ftp, php_socket_t s)
 	return 1;
 }
 /* }}} */
-/* {{{ data_writeable
- */
+/* {{{ data_writeable */
 int
 data_writeable(ftpbuf_t *ftp, php_socket_t s)
 {
@@ -1628,8 +1587,7 @@ data_writeable(ftpbuf_t *ftp, php_socket_t s)
 }
 /* }}} */
 
-/* {{{ my_accept
- */
+/* {{{ my_accept */
 int
 my_accept(ftpbuf_t *ftp, php_socket_t s, struct sockaddr *addr, socklen_t *addrlen)
 {
@@ -1653,8 +1611,7 @@ my_accept(ftpbuf_t *ftp, php_socket_t s, struct sockaddr *addr, socklen_t *addrl
 }
 /* }}} */
 
-/* {{{ ftp_getdata
- */
+/* {{{ ftp_getdata */
 databuf_t*
 ftp_getdata(ftpbuf_t *ftp)
 {
@@ -1731,7 +1688,7 @@ ftp_getdata(ftpbuf_t *ftp)
 
 	data->listener = fd;
 
-#if HAVE_IPV6 && HAVE_INET_NTOP
+#if defined(HAVE_IPV6) && defined(HAVE_INET_NTOP)
 	if (sa->sa_family == AF_INET6) {
 		/* need to use EPRT */
 		char eprtarg[INET6_ADDRSTRLEN + sizeof("|x||xxxxx|")];
@@ -1784,8 +1741,7 @@ bail:
 }
 /* }}} */
 
-/* {{{ data_accept
- */
+/* {{{ data_accept */
 databuf_t*
 data_accept(databuf_t *data, ftpbuf_t *ftp)
 {
@@ -1897,8 +1853,7 @@ data_accepted:
 }
 /* }}} */
 
-/* {{{ ftp_ssl_shutdown
- */
+/* {{{ ftp_ssl_shutdown */
 #ifdef HAVE_FTP_SSL
 static void ftp_ssl_shutdown(ftpbuf_t *ftp, php_socket_t fd, SSL *ssl_handle) {
 	/* In TLS 1.3 it's common to receive session tickets after the handshake has completed. We need to train
@@ -1961,8 +1916,7 @@ static void ftp_ssl_shutdown(ftpbuf_t *ftp, php_socket_t fd, SSL *ssl_handle) {
 #endif
 /* }}} */
 
-/* {{{ data_close
- */
+/* {{{ data_close */
 databuf_t*
 data_close(ftpbuf_t *ftp, databuf_t *data)
 {
@@ -1997,8 +1951,7 @@ data_close(ftpbuf_t *ftp, databuf_t *data)
 }
 /* }}} */
 
-/* {{{ ftp_genlist
- */
+/* {{{ ftp_genlist */
 char**
 ftp_genlist(ftpbuf_t *ftp, const char *cmd, const size_t cmd_len, const char *path, const size_t path_len)
 {
@@ -2102,8 +2055,7 @@ bail:
 }
 /* }}} */
 
-/* {{{ ftp_nb_get
- */
+/* {{{ ftp_nb_get */
 int
 ftp_nb_get(ftpbuf_t *ftp, php_stream *outstream, const char *path, const size_t path_len, ftptype_t type, zend_long resumepos)
 {
@@ -2160,8 +2112,7 @@ bail:
 }
 /* }}} */
 
-/* {{{ ftp_nb_continue_read
- */
+/* {{{ ftp_nb_continue_read */
 int
 ftp_nb_continue_read(ftpbuf_t *ftp)
 {
@@ -2223,8 +2174,7 @@ bail:
 }
 /* }}} */
 
-/* {{{ ftp_nb_put
- */
+/* {{{ ftp_nb_put */
 int
 ftp_nb_put(ftpbuf_t *ftp, const char *path, const size_t path_len, php_stream *instream, ftptype_t type, zend_long startpos)
 {
@@ -2277,8 +2227,7 @@ bail:
 /* }}} */
 
 
-/* {{{ ftp_nb_continue_write
- */
+/* {{{ ftp_nb_continue_write */
 int
 ftp_nb_continue_write(ftpbuf_t *ftp)
 {
@@ -2328,5 +2277,3 @@ bail:
 	return PHP_FTP_FAILED;
 }
 /* }}} */
-
-#endif /* HAVE_FTP */
