@@ -1,6 +1,4 @@
 /*
-   +----------------------------------------------------------------------+
-   | PHP Version 7                                                        |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -52,7 +50,6 @@ static const size_t DEF_UTF16_BUF_SIZE = 1024;
 /* {{{ collator_regular_compare_function */
 static int collator_regular_compare_function(zval *result, zval *op1, zval *op2)
 {
-	Collator_object* co = NULL;
 	int rc = SUCCESS;
 	zval str1, str2;
 	zval num1, num2;
@@ -72,20 +69,10 @@ static int collator_regular_compare_function(zval *result, zval *op1, zval *op2)
 		( str1_p == ( num1_p = collator_convert_string_to_number_if_possible( str1_p, &num1 ) ) ||
 		  str2_p == ( num2_p = collator_convert_string_to_number_if_possible( str2_p, &num2 ) ) ) )
 	{
-		/* Fetch collator object. */
-		co = Z_INTL_COLLATOR_P(&INTL_G(current_collator));
-
-		if (!co || !co->ucoll) {
-			intl_error_set_code( NULL, COLLATOR_ERROR_CODE( co ) );
-			intl_errors_set_custom_msg( COLLATOR_ERROR_P( co ),
-				"Object not initialized", 0 );
-			php_error_docref(NULL, E_RECOVERABLE_ERROR, "Object not initialized");
-
-		}
-
 		/* Compare the strings using ICU. */
+		ZEND_ASSERT(INTL_G(current_collator) != NULL);
 		ZVAL_LONG(result, ucol_strcoll(
-					co->ucoll,
+					INTL_G(current_collator),
 					INTL_Z_STRVAL_P(str1_p), INTL_Z_STRLEN_P(str1_p),
 					INTL_Z_STRVAL_P(str2_p), INTL_Z_STRLEN_P(str2_p) ));
 	}
@@ -182,19 +169,16 @@ static int collator_icu_compare_function(zval *result, zval *op1, zval *op2)
 {
 	zval str1, str2;
 	int rc              = SUCCESS;
-	Collator_object* co = NULL;
 	zval *str1_p        = NULL;
 	zval *str2_p        = NULL;
 
 	str1_p = collator_make_printable_zval( op1, &str1);
 	str2_p = collator_make_printable_zval( op2, &str2 );
 
-	/* Fetch collator object. */
-	co = Z_INTL_COLLATOR_P(&INTL_G(current_collator));
-
 	/* Compare the strings using ICU. */
+	ZEND_ASSERT(INTL_G(current_collator) != NULL);
 	ZVAL_LONG(result, ucol_strcoll(
-				co->ucoll,
+				INTL_G(current_collator),
 				INTL_Z_STRVAL_P(str1_p), INTL_Z_STRLEN_P(str1_p),
 				INTL_Z_STRVAL_P(str2_p), INTL_Z_STRLEN_P(str2_p) ));
 
@@ -208,19 +192,11 @@ static int collator_icu_compare_function(zval *result, zval *op1, zval *op2)
 /* {{{ collator_compare_func
  * Taken from PHP7 source (array_data_compare).
  */
-static int collator_compare_func( const void* a, const void* b )
+static int collator_compare_func(Bucket *f, Bucket *s)
 {
-	Bucket *f;
-	Bucket *s;
 	zval result;
-	zval *first;
-	zval *second;
-
-	f = (Bucket *) a;
-	s = (Bucket *) b;
-
-	first = &f->val;
-	second = &s->val;
+	zval *first = &f->val;
+	zval *second = &s->val;
 
 	if( INTL_G(compare_func)( &result, first, second) == FAILURE )
 		return 0;
@@ -246,9 +222,7 @@ static int collator_compare_func( const void* a, const void* b )
 }
 /* }}} */
 
-/* {{{ collator_cmp_sort_keys
- * Compare sort keys
- */
+/* {{{ Compare sort keys */
 static int collator_cmp_sort_keys( const void *p1, const void *p2 )
 {
 	char* key1 = ((collator_sort_key_index_t*)p1)->key;
@@ -258,9 +232,7 @@ static int collator_cmp_sort_keys( const void *p1, const void *p2 )
 }
 /* }}} */
 
-/* {{{ collator_get_compare_function
- * Choose compare function according to sort flags.
- */
+/* {{{ Choose compare function according to sort flags. */
 static collator_compare_func_t collator_get_compare_function( const zend_long sort_flags )
 {
 	collator_compare_func_t func;
@@ -285,12 +257,10 @@ static collator_compare_func_t collator_get_compare_function( const zend_long so
 }
 /* }}} */
 
-/* {{{ collator_sort_internal
- * Common code shared by collator_sort() and collator_asort() API functions.
- */
+/* {{{ Common code shared by collator_sort() and collator_asort() API functions. */
 static void collator_sort_internal( int renumber, INTERNAL_FUNCTION_PARAMETERS )
 {
-	zval           saved_collator;
+	UCollator*     saved_collator;
 	zval*          array            = NULL;
 	HashTable*     hash             = NULL;
 	zend_long           sort_flags       = COLLATOR_SORT_REGULAR;
@@ -301,15 +271,17 @@ static void collator_sort_internal( int renumber, INTERNAL_FUNCTION_PARAMETERS )
 	if( zend_parse_method_parameters( ZEND_NUM_ARGS(), getThis(), "Oa/|l",
 		&object, Collator_ce_ptr, &array, &sort_flags ) == FAILURE )
 	{
-		intl_error_set( NULL, U_ILLEGAL_ARGUMENT_ERROR,
-			"collator_sort_internal: unable to parse input params", 0 );
-
-		RETURN_FALSE;
+		RETURN_THROWS();
 	}
 
 	/* Fetch the object. */
 	COLLATOR_METHOD_FETCH_OBJECT;
 
+	if (!co->ucoll) {
+		zend_throw_error(NULL, "Object not initialized");
+		RETURN_THROWS();
+	}
+
 	/* Set 'compare function' according to sort flags. */
 	INTL_G(compare_func) = collator_get_compare_function( sort_flags );
 
@@ -320,14 +292,14 @@ static void collator_sort_internal( int renumber, INTERNAL_FUNCTION_PARAMETERS )
 	COLLATOR_CHECK_STATUS( co, "Error converting hash from UTF-8 to UTF-16" );
 
 	/* Save specified collator in the request-global (?) variable. */
-	ZVAL_COPY_VALUE(&saved_collator, &INTL_G( current_collator ));
-	ZVAL_COPY_VALUE(&INTL_G( current_collator ), object);
+	saved_collator = INTL_G( current_collator );
+	INTL_G( current_collator ) = co->ucoll;
 
 	/* Sort specified array. */
 	zend_hash_sort(hash, collator_compare_func, renumber);
 
 	/* Restore saved collator. */
-	ZVAL_COPY_VALUE(&INTL_G( current_collator ), &saved_collator);
+	INTL_G( current_collator ) = saved_collator;
 
 	/* Convert strings in the specified array back to UTF-8. */
 	collator_convert_hash_from_utf16_to_utf8( hash, COLLATOR_ERROR_CODE_P( co ) );
@@ -337,14 +309,10 @@ static void collator_sort_internal( int renumber, INTERNAL_FUNCTION_PARAMETERS )
 }
 /* }}} */
 
-/* {{{ proto bool Collator::sort( Collator $coll, array(string) $arr [, int $sort_flags] )
- * Sort array using specified collator. }}} */
-/* {{{ proto bool collator_sort(  Collator $coll, array(string) $arr [, int $sort_flags] )
- * Sort array using specified collator.
- */
+/* {{{ Sort array using specified collator. */
 PHP_FUNCTION( collator_sort )
 {
-	collator_sort_internal( TRUE, INTERNAL_FUNCTION_PARAM_PASSTHRU );
+	collator_sort_internal( true, INTERNAL_FUNCTION_PARAM_PASSTHRU );
 }
 /* }}} */
 
@@ -357,11 +325,7 @@ static void collator_sortkey_swap(collator_sort_key_index_t *p, collator_sort_ke
 }
 /* }}} */
 
-/* {{{ proto bool Collator::sortWithSortKeys( Collator $coll, array(string) $arr )
- * Equivalent to standard PHP sort using Collator.
- * Uses ICU ucol_getSortKey for performance. }}} */
-/* {{{ proto bool collator_sort_with_sort_keys( Collator $coll, array(string) $arr )
- * Equivalent to standard PHP sort using Collator.
+/* {{{ Equivalent to standard PHP sort using Collator.
  * Uses ICU ucol_getSortKey for performance.
  */
 PHP_FUNCTION( collator_sort_with_sort_keys )
@@ -374,7 +338,7 @@ PHP_FUNCTION( collator_sort_with_sort_keys )
 	char*       sortKeyBuf           = NULL;                     /* buffer to store sort keys */
 	uint32_t    sortKeyBufSize       = DEF_SORT_KEYS_BUF_SIZE;   /* buffer size */
 	ptrdiff_t   sortKeyBufOffset     = 0;                        /* pos in buffer to store sort key */
-	int32_t     sortKeyLen           = 0;                        /* the length of currently processing key */
+	uint32_t    sortKeyLen           = 0;                        /* the length of currently processing key */
 	uint32_t    bufLeft              = 0;
 	uint32_t    bufIncrement         = 0;
 
@@ -395,10 +359,7 @@ PHP_FUNCTION( collator_sort_with_sort_keys )
 	if( zend_parse_method_parameters( ZEND_NUM_ARGS(), getThis(), "Oa",
 		&object, Collator_ce_ptr, &array ) == FAILURE )
 	{
-		intl_error_set( NULL, U_ILLEGAL_ARGUMENT_ERROR,
-			"collator_sort_with_sort_keys: unable to parse input params", 0 );
-
-		RETURN_FALSE;
+		RETURN_THROWS();
 	}
 
 	/* Fetch the object. */
@@ -408,9 +369,9 @@ PHP_FUNCTION( collator_sort_with_sort_keys )
 		intl_error_set_code( NULL, COLLATOR_ERROR_CODE( co ) );
 		intl_errors_set_custom_msg( COLLATOR_ERROR_P( co ),
 			"Object not initialized", 0 );
-		php_error_docref(NULL, E_RECOVERABLE_ERROR, "Object not initialized");
+		zend_throw_error(NULL, "Object not initialized");
 
-		RETURN_FALSE;
+		RETURN_THROWS();
 	}
 
 	/*
@@ -531,21 +492,14 @@ PHP_FUNCTION( collator_sort_with_sort_keys )
 }
 /* }}} */
 
-/* {{{ proto bool Collator::asort( Collator $coll, array(string) $arr )
- * Sort array using specified collator, maintaining index association. }}} */
-/* {{{ proto bool collator_asort( Collator $coll, array(string) $arr )
- * Sort array using specified collator, maintaining index association.
- */
+/* {{{ Sort array using specified collator, maintaining index association. */
 PHP_FUNCTION( collator_asort )
 {
-	collator_sort_internal( FALSE, INTERNAL_FUNCTION_PARAM_PASSTHRU );
+	collator_sort_internal( false, INTERNAL_FUNCTION_PARAM_PASSTHRU );
 }
 /* }}} */
 
-/* {{{ proto bool Collator::getSortKey( Collator $coll, string $str )
- * Get a sort key for a string from a Collator. }}} */
-/* {{{ proto bool collator_get_sort_key( Collator $coll, string $str )
- * Get a sort key for a string from a Collator. */
+/* {{{ Get a sort key for a string from a Collator. */
 PHP_FUNCTION( collator_get_sort_key )
 {
 	char*            str      = NULL;
@@ -561,10 +515,7 @@ PHP_FUNCTION( collator_get_sort_key )
 	if( zend_parse_method_parameters( ZEND_NUM_ARGS(), getThis(), "Os",
 		&object, Collator_ce_ptr, &str, &str_len ) == FAILURE )
 	{
-		intl_error_set( NULL, U_ILLEGAL_ARGUMENT_ERROR,
-			 "collator_get_sort_key: unable to parse input params", 0 );
-
-		RETURN_FALSE;
+		RETURN_THROWS();
 	}
 
 	/* Fetch the object. */
@@ -574,9 +525,9 @@ PHP_FUNCTION( collator_get_sort_key )
 		intl_error_set_code( NULL, COLLATOR_ERROR_CODE( co ) );
 		intl_errors_set_custom_msg( COLLATOR_ERROR_P( co ),
 			"Object not initialized", 0 );
-		php_error_docref(NULL, E_RECOVERABLE_ERROR, "Object not initialized");
+		zend_throw_error(NULL, "Object not initialized");
 
-		RETURN_FALSE;
+		RETURN_THROWS();
 	}
 
 	/*
@@ -615,12 +566,3 @@ PHP_FUNCTION( collator_get_sort_key )
 	RETVAL_NEW_STR(key_str);
 }
 /* }}} */
-
-/*
- * Local variables:
- * tab-width: 4
- * c-basic-offset: 4
- * End:
- * vim600: noet sw=4 ts=4 fdm=marker
- * vim<600: noet sw=4 ts=4
- */
