@@ -1,6 +1,4 @@
 /*
-   +----------------------------------------------------------------------+
-   | PHP Version 7                                                        |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -32,12 +30,9 @@ using icu::Locale;
 using icu::Calendar;
 using icu::DateFormat;
 
-int datefmt_process_calendar_arg(zval* calendar_zv,
-								 Locale const& locale,
-								 const char *func_name,
-								 intl_error *err,
-								 Calendar*& cal,
-								 zend_long& cal_int_type,
-								 bool& calendar_owned);
+int datefmt_process_calendar_arg(
+	zend_object *calendar_obj, zend_long calendar_long, zend_bool calendar_is_null, Locale const& locale,
+	const char *func_name, intl_error *err, Calendar*& cal, zend_long& cal_int_type, bool& calendar_owned
+);
 
 #endif	/* DATEFORMAT_HELPERS_H */
