@@ -1,6 +1,4 @@
 /*
-   +----------------------------------------------------------------------+
-   | PHP Version 7                                                        |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -22,7 +20,6 @@
 
 #include "php_intl.h"
 #include "msgformat_class.h"
-#include "msgformat_parse.h"
 #include "msgformat_data.h"
 #include "msgformat_helpers.h"
 #include "intl_convert.h"
@@ -53,11 +50,7 @@ static void msgfmt_do_parse(MessageFormatter_object *mfo, char *source, size_t s
 }
 /* }}} */
 
-/* {{{ proto array MessageFormatter::parse( string $source )
- * Parse a message }}} */
-/* {{{ proto array msgfmt_parse( MessageFormatter $nf, string $source )
- * Parse a message.
- */
+/* {{{ Parse a message */
 PHP_FUNCTION( msgfmt_parse )
 {
 	char *source;
@@ -69,10 +62,7 @@ PHP_FUNCTION( msgfmt_parse )
 	if( zend_parse_method_parameters( ZEND_NUM_ARGS(), getThis(), "Os",
 		&object, MessageFormatter_ce_ptr,  &source, &source_len ) == FAILURE )
 	{
-		intl_error_set( NULL, U_ILLEGAL_ARGUMENT_ERROR,
-			"msgfmt_parse: unable to parse input params", 0 );
-
-		RETURN_FALSE;
+		RETURN_THROWS();
 	}
 
 	/* Fetch the object. */
@@ -82,11 +72,7 @@ PHP_FUNCTION( msgfmt_parse )
 }
 /* }}} */
 
-/* {{{ proto array MessageFormatter::formatMessage( string $locale, string $pattern, string $source )
- * Parse a message. }}} */
-/* {{{ proto array numfmt_parse_message( string $locale, string $pattern, string $source )
- * Parse a message.
- */
+/* {{{ Parse a message. */
 PHP_FUNCTION( msgfmt_parse_message )
 {
 	UChar      *spattern = NULL;
@@ -104,10 +90,7 @@ PHP_FUNCTION( msgfmt_parse_message )
 	if( zend_parse_parameters( ZEND_NUM_ARGS(), "sss",
 		  &slocale, &slocale_len, &pattern, &pattern_len, &source, &src_len ) == FAILURE )
 	{
-		intl_error_set( NULL, U_ILLEGAL_ARGUMENT_ERROR,
-			"msgfmt_parse_message: unable to parse input params", 0 );
-
-		RETURN_FALSE;
+		RETURN_THROWS();
 	}
 
 	INTL_CHECK_LOCALE_LEN(slocale_len);
@@ -152,12 +135,3 @@ PHP_FUNCTION( msgfmt_parse_message )
 	msgformat_data_free(&mfo->mf_data);
 }
 /* }}} */
-
-/*
- * Local variables:
- * tab-width: 4
- * c-basic-offset: 4
- * End:
- * vim600: noet sw=4 ts=4 fdm=marker
- * vim<600: noet sw=4 ts=4
- */
