@@ -27,10 +27,6 @@
  *
  */
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
 #include "mbfilter.h"
 #include "mbfilter_hz.h"
 
@@ -44,13 +40,14 @@ const mbfl_encoding mbfl_encoding_hz = {
 	"HZ-GB-2312",
 	NULL,
 	NULL,
-	MBFL_ENCTYPE_MBCS | MBFL_ENCTYPE_SHFTCODE | MBFL_ENCTYPE_GL_UNSAFE
+	MBFL_ENCTYPE_MBCS | MBFL_ENCTYPE_GL_UNSAFE,
+	&vtbl_hz_wchar,
+	&vtbl_wchar_hz
 };
 
 const struct mbfl_identify_vtbl vtbl_identify_hz = {
 	mbfl_no_encoding_hz,
 	mbfl_filt_ident_common_ctor,
-	mbfl_filt_ident_common_dtor,
 	mbfl_filt_ident_hz
 };
 
@@ -58,18 +55,20 @@ const struct mbfl_convert_vtbl vtbl_hz_wchar = {
 	mbfl_no_encoding_hz,
 	mbfl_no_encoding_wchar,
 	mbfl_filt_conv_common_ctor,
-	mbfl_filt_conv_common_dtor,
+	NULL,
 	mbfl_filt_conv_hz_wchar,
-	mbfl_filt_conv_common_flush
+	mbfl_filt_conv_common_flush,
+	NULL,
 };
 
 const struct mbfl_convert_vtbl vtbl_wchar_hz = {
 	mbfl_no_encoding_wchar,
 	mbfl_no_encoding_hz,
 	mbfl_filt_conv_common_ctor,
-	mbfl_filt_conv_common_dtor,
+	NULL,
 	mbfl_filt_conv_wchar_hz,
-	mbfl_filt_conv_any_hz_flush
+	mbfl_filt_conv_any_hz_flush,
+	NULL,
 };
 
 #define CK(statement)	do { if ((statement) < 0) return (-1); } while (0)
@@ -209,9 +208,7 @@ mbfl_filt_conv_wchar_hz(int c, mbfl_convert_filter *filter)
 			CK((*filter->output_function)(s & 0x7f, filter->data));
 		}
 	} else {
-		if (filter->illegal_mode != MBFL_OUTPUTFILTER_ILLEGAL_MODE_NONE) {
-			CK(mbfl_filt_conv_illegal_output(c, filter));
-		}
+		CK(mbfl_filt_conv_illegal_output(c, filter));
 	}
 
 	return c;
@@ -274,11 +271,3 @@ static int mbfl_filt_ident_hz(int c, mbfl_identify_filter *filter)
 
 	return c;
 }
-
-
-/*
- * Local variables:
- * tab-width: 4
- * c-basic-offset: 4
- * End:
- */
