@@ -28,65 +28,66 @@
  */
 
 #include "mbfilter.h"
-#include "mbfilter_iso8859_10.h"
-#include "unicode_table_iso8859_10.h"
+#include "mbfilter_iso8859_6.h"
+#include "unicode_table_iso8859_6.h"
 
-static const char *mbfl_encoding_8859_10_aliases[] = {"ISO8859-10", "latin6", NULL};
+static const char *mbfl_encoding_8859_6_aliases[] = {"ISO8859-6", "arabic", NULL};
 
-const mbfl_encoding mbfl_encoding_8859_10 = {
-	mbfl_no_encoding_8859_10,
-	"ISO-8859-10",
-	"ISO-8859-10",
-	(const char *(*)[])&mbfl_encoding_8859_10_aliases,
+const mbfl_encoding mbfl_encoding_8859_6 = {
+	mbfl_no_encoding_8859_6,
+	"ISO-8859-6",
+	"ISO-8859-6",
+	(const char *(*)[])&mbfl_encoding_8859_6_aliases,
 	NULL,
 	MBFL_ENCTYPE_SBCS,
-	&vtbl_8859_10_wchar,
-	&vtbl_wchar_8859_10
+	&vtbl_8859_6_wchar,
+	&vtbl_wchar_8859_6
 };
 
-const struct mbfl_identify_vtbl vtbl_identify_8859_10 = {
-	mbfl_no_encoding_8859_10,
+const struct mbfl_identify_vtbl vtbl_identify_8859_6 = {
+	mbfl_no_encoding_8859_6,
 	mbfl_filt_ident_common_ctor,
 	mbfl_filt_ident_true
 };
 
-const struct mbfl_convert_vtbl vtbl_8859_10_wchar = {
-	mbfl_no_encoding_8859_10,
+const struct mbfl_convert_vtbl vtbl_8859_6_wchar = {
+	mbfl_no_encoding_8859_6,
 	mbfl_no_encoding_wchar,
 	mbfl_filt_conv_common_ctor,
 	NULL,
-	mbfl_filt_conv_8859_10_wchar,
+	mbfl_filt_conv_8859_6_wchar,
 	mbfl_filt_conv_common_flush,
 	NULL,
 };
 
-const struct mbfl_convert_vtbl vtbl_wchar_8859_10 = {
+const struct mbfl_convert_vtbl vtbl_wchar_8859_6 = {
 	mbfl_no_encoding_wchar,
-	mbfl_no_encoding_8859_10,
+	mbfl_no_encoding_8859_6,
 	mbfl_filt_conv_common_ctor,
 	NULL,
-	mbfl_filt_conv_wchar_8859_10,
+	mbfl_filt_conv_wchar_8859_6,
 	mbfl_filt_conv_common_flush,
 	NULL,
 };
 
+
 #define CK(statement)	do { if ((statement) < 0) return (-1); } while (0)
 
 /*
- * ISO-8859-10 => wchar
+ * ISO-8859-6 => wchar
  */
-int mbfl_filt_conv_8859_10_wchar(int c, mbfl_convert_filter *filter)
+int mbfl_filt_conv_8859_6_wchar(int c, mbfl_convert_filter *filter)
 {
 	int s;
 
 	if (c >= 0 && c < 0xa0) {
 		s = c;
 	} else if (c >= 0xa0 && c < 0x100) {
-		s = iso8859_10_ucs_table[c - 0xa0];
+		s = iso8859_6_ucs_table[c - 0xa0];
 		if (s <= 0) {
 			s = c;
 			s &= MBFL_WCSPLANE_MASK;
-			s |= MBFL_WCSPLANE_8859_10;
+			s |= MBFL_WCSPLANE_8859_6;
 		}
 	} else {
 		s = c;
@@ -100,9 +101,9 @@ int mbfl_filt_conv_8859_10_wchar(int c, mbfl_convert_filter *filter)
 }
 
 /*
- * wchar => ISO-8859-10
+ * wchar => ISO-8859-6
  */
-int mbfl_filt_conv_wchar_8859_10(int c, mbfl_convert_filter *filter)
+int mbfl_filt_conv_wchar_8859_6(int c, mbfl_convert_filter *filter)
 {
 	int s, n;
 
@@ -112,13 +113,13 @@ int mbfl_filt_conv_wchar_8859_10(int c, mbfl_convert_filter *filter)
 		s = -1;
 		n = 95;
 		while (n >= 0) {
-			if (c == iso8859_10_ucs_table[n]) {
+			if (c == iso8859_6_ucs_table[n]) {
 				s = 0xa0 + n;
 				break;
 			}
 			n--;
 		}
-		if (s <= 0 && (c & ~MBFL_WCSPLANE_MASK) == MBFL_WCSPLANE_8859_10) {
+		if (s <= 0 && (c & ~MBFL_WCSPLANE_MASK) == MBFL_WCSPLANE_8859_6) {
 			s = c & MBFL_WCSPLANE_MASK;
 		}
 	}
