@@ -27,10 +27,6 @@
  *
  */
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
 #include "mbfilter.h"
 #include "mbfilter_utf7.h"
 
@@ -57,7 +53,7 @@ const mbfl_encoding mbfl_encoding_utf7 = {
 	"UTF-7",
 	(const char *(*)[])&mbfl_encoding_utf7_aliases,
 	NULL,
-	MBFL_ENCTYPE_MBCS | MBFL_ENCTYPE_SHFTCODE | MBFL_ENCTYPE_GL_UNSAFE,
+	MBFL_ENCTYPE_MBCS | MBFL_ENCTYPE_GL_UNSAFE,
 	&vtbl_utf7_wchar,
 	&vtbl_wchar_utf7
 };
@@ -65,7 +61,6 @@ const mbfl_encoding mbfl_encoding_utf7 = {
 const struct mbfl_identify_vtbl vtbl_identify_utf7 = {
 	mbfl_no_encoding_utf7,
 	mbfl_filt_ident_common_ctor,
-	mbfl_filt_ident_common_dtor,
 	mbfl_filt_ident_utf7
 };
 
@@ -73,18 +68,20 @@ const struct mbfl_convert_vtbl vtbl_utf7_wchar = {
 	mbfl_no_encoding_utf7,
 	mbfl_no_encoding_wchar,
 	mbfl_filt_conv_common_ctor,
-	mbfl_filt_conv_common_dtor,
+	NULL,
 	mbfl_filt_conv_utf7_wchar,
-	mbfl_filt_conv_common_flush
+	mbfl_filt_conv_common_flush,
+	NULL,
 };
 
 const struct mbfl_convert_vtbl vtbl_wchar_utf7 = {
 	mbfl_no_encoding_wchar,
 	mbfl_no_encoding_utf7,
 	mbfl_filt_conv_common_ctor,
-	mbfl_filt_conv_common_dtor,
+	NULL,
 	mbfl_filt_conv_wchar_utf7,
-	mbfl_filt_conv_wchar_utf7_flush
+	mbfl_filt_conv_wchar_utf7_flush,
+	NULL,
 };
 
 
