@@ -28,15 +28,10 @@
  *
  */
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
 #include <stddef.h>
-
 #include <string.h>
-#include "mbfl_allocators.h"
-#include "mbfl_string.h"
+
+#include "zend.h"
 #include "mbfl_memory_device.h"
 
 /*
@@ -49,10 +44,8 @@ mbfl_memory_device_init(mbfl_memory_device *device, size_t initsz, size_t allocs
 		device->length = 0;
 		device->buffer = NULL;
 		if (initsz > 0) {
-			device->buffer = (unsigned char *)mbfl_malloc(initsz);
-			if (device->buffer != NULL) {
-				device->length = initsz;
-			}
+			device->buffer = emalloc(initsz);
+			device->length = initsz;
 		}
 		device->pos = 0;
 		if (allocsz > MBFL_MEMORY_DEVICE_ALLOC_SIZE) {
@@ -66,15 +59,10 @@ mbfl_memory_device_init(mbfl_memory_device *device, size_t initsz, size_t allocs
 void
 mbfl_memory_device_realloc(mbfl_memory_device *device, size_t initsz, size_t allocsz)
 {
-	unsigned char *tmp;
-
 	if (device) {
 		if (initsz > device->length) {
-			tmp = (unsigned char *)mbfl_realloc((void *)device->buffer, initsz);
-			if (tmp != NULL) {
-				device->buffer = tmp;
-				device->length = initsz;
-			}
+			device->buffer = erealloc(device->buffer, initsz);
+			device->length = initsz;
 		}
 		if (allocsz > MBFL_MEMORY_DEVICE_ALLOC_SIZE) {
 			device->allocsz = allocsz;
@@ -89,7 +77,7 @@ mbfl_memory_device_clear(mbfl_memory_device *device)
 {
 	if (device) {
 		if (device->buffer) {
-			mbfl_free(device->buffer);
+			efree(device->buffer);
 		}
 		device->buffer = NULL;
 		device->length = 0;
@@ -142,7 +130,6 @@ mbfl_memory_device_output(int c, void *data)
 	if (device->pos >= device->length) {
 		/* reallocate buffer */
 		size_t newlen;
-		unsigned char *tmp;
 
 		if (device->length > SIZE_MAX - device->allocsz) {
 			/* overflow */
@@ -150,80 +137,14 @@ mbfl_memory_device_output(int c, void *data)
 		}
 
 		newlen = device->length + device->allocsz;
-		tmp = (unsigned char *)mbfl_realloc((void *)device->buffer, newlen);
-		if (tmp == NULL) {
-			return -1;
-		}
+		device->buffer = erealloc(device->buffer, newlen);
 		device->length = newlen;
-		device->buffer = tmp;
 	}
 
 	device->buffer[device->pos++] = (unsigned char)c;
 	return c;
 }
 
-int
-mbfl_memory_device_output2(int c, void *data)
-{
-	mbfl_memory_device *device = (mbfl_memory_device *)data;
-
-	if (2 > device->length - device->pos) {
-		/* reallocate buffer */
-		size_t newlen;
-		unsigned char *tmp;
-
-		if (device->length > SIZE_MAX - device->allocsz) {
-			/* overflow */
-			return -1;
-		}
-
-		newlen = device->length + device->allocsz;
-		tmp = (unsigned char *)mbfl_realloc((void *)device->buffer, newlen);
-		if (tmp == NULL) {
-			return -1;
-		}
-		device->length = newlen;
-		device->buffer = tmp;
-	}
-
-	device->buffer[device->pos++] = (unsigned char)((c >> 8) & 0xff);
-	device->buffer[device->pos++] = (unsigned char)(c & 0xff);
-
-	return c;
-}
-
-int
-mbfl_memory_device_output4(int c, void* data)
-{
-	mbfl_memory_device *device = (mbfl_memory_device *)data;
-
-	if (4 > device->length - device->pos) {
-		/* reallocate buffer */
-		size_t newlen;
-		unsigned char *tmp;
-
-		if (device->length > SIZE_MAX - device->allocsz) {
-			/* overflow */
-			return -1;
-		}
-
-		newlen = device->length + device->allocsz;
-		tmp = (unsigned char *)mbfl_realloc((void *)device->buffer, newlen);
-		if (tmp == NULL) {
-			return -1;
-		}
-		device->length = newlen;
-		device->buffer = tmp;
-	}
-
-	device->buffer[device->pos++] = (unsigned char)((c >> 24) & 0xff);
-	device->buffer[device->pos++] = (unsigned char)((c >> 16) & 0xff);
-	device->buffer[device->pos++] = (unsigned char)((c >> 8) & 0xff);
-	device->buffer[device->pos++] = (unsigned char)(c & 0xff);
-
-	return c;
-}
-
 int
 mbfl_memory_device_strcat(mbfl_memory_device *device, const char *psrc)
 {
@@ -238,7 +159,6 @@ mbfl_memory_device_strncat(mbfl_memory_device *device, const char *psrc, size_t
 	if (len > device->length - device->pos) {
 		/* reallocate buffer */
 		size_t newlen;
-		unsigned char *tmp;
 
 		if (len > SIZE_MAX - MBFL_MEMORY_DEVICE_ALLOC_SIZE
 				|| device->length > SIZE_MAX - (len + MBFL_MEMORY_DEVICE_ALLOC_SIZE)) {
@@ -247,13 +167,8 @@ mbfl_memory_device_strncat(mbfl_memory_device *device, const char *psrc, size_t
 		}
 
 		newlen = device->length + len + MBFL_MEMORY_DEVICE_ALLOC_SIZE;
-		tmp = (unsigned char *)mbfl_realloc((void *)device->buffer, newlen);
-		if (tmp == NULL) {
-			return -1;
-		}
-
+		device->buffer = erealloc(device->buffer, newlen);
 		device->length = newlen;
-		device->buffer = tmp;
 	}
 
 	w = &device->buffer[device->pos];
@@ -285,7 +200,7 @@ mbfl_wchar_device_clear(mbfl_wchar_device *device)
 {
 	if (device) {
 		if (device->buffer) {
-			mbfl_free(device->buffer);
+			efree(device->buffer);
 		}
 		device->buffer = NULL;
 		device->length = 0;
@@ -301,7 +216,6 @@ mbfl_wchar_device_output(int c, void *data)
 	if (device->pos >= device->length) {
 		/* reallocate buffer */
 		size_t newlen;
-		unsigned int *tmp;
 
 		if (device->length > SIZE_MAX - device->allocsz) {
 			/* overflow */
@@ -314,12 +228,8 @@ mbfl_wchar_device_output(int c, void *data)
 			return -1;
 		}
 
-		tmp = (unsigned int *)mbfl_realloc((void *)device->buffer, newlen*sizeof(int));
-		if (tmp == NULL) {
-			return -1;
-		}
+		device->buffer = erealloc(device->buffer, newlen*sizeof(int));
 		device->length = newlen;
-		device->buffer = tmp;
 	}
 
 	device->buffer[device->pos++] = c;
