@@ -1,6 +1,4 @@
 /*
-  +----------------------------------------------------------------------+
-  | PHP Version 7                                                        |
   +----------------------------------------------------------------------+
   | Copyright (c) The PHP Group                                          |
   +----------------------------------------------------------------------+
@@ -27,7 +25,6 @@
 #include "ext/standard/info.h"
 #include "php_mysqli_structs.h"
 #include "zend_exceptions.h"
-#include "mysqli_fe.h"
 
 #define MAP_PROPERTY_MYG_BOOL_READ(name, value) \
 static int name(mysqli_object *obj, zval *retval, zend_bool quiet) \
@@ -81,15 +78,6 @@ static int driver_report_write(mysqli_object *obj, zval *value)
 }
 /* }}} */
 
-/* {{{ property driver_embedded_read */
-static int driver_embedded_read(mysqli_object *obj, zval *retval, zend_bool quiet)
-{
-	ZVAL_FALSE(retval);
-
-	return SUCCESS;
-}
-/* }}} */
-
 /* {{{ property driver_client_version_read */
 static int driver_client_version_read(mysqli_object *obj, zval *retval, zend_bool quiet)
 {
@@ -123,7 +111,7 @@ MAP_PROPERTY_MYG_LONG_READ(driver_report_read, report_mode)
 
 ZEND_FUNCTION(mysqli_driver_construct)
 {
-#if G0
+#ifdef G0
 	MYSQLI_RESOURCE 	*mysqli_resource;
 
 	mysqli_resource = (MYSQLI_RESOURCE *)ecalloc (1, sizeof(MYSQLI_RESOURCE));
@@ -137,15 +125,8 @@ const mysqli_property_entry mysqli_driver_property_entries[] = {
 	{"client_info", sizeof("client_info") - 1, driver_client_info_read, NULL},
 	{"client_version", sizeof("client_version") - 1, driver_client_version_read, NULL},
 	{"driver_version", sizeof("driver_version") - 1, driver_driver_version_read, NULL},
-	{"embedded", sizeof("embedded") - 1, driver_embedded_read, NULL},
 	{"reconnect", sizeof("reconnect") - 1, driver_reconnect_read, driver_reconnect_write},
 	{"report_mode", sizeof("report_mode") - 1, driver_report_read, driver_report_write},
 	{NULL, 0, NULL, NULL}
 };
 
-/* {{{ mysqli_driver_methods[]
- */
-const zend_function_entry mysqli_driver_methods[] = {
-	PHP_FE_END
-};
-/* }}} */
