@@ -1,6 +1,4 @@
 /*
-   +----------------------------------------------------------------------+
-   | PHP Version 7                                                        |
    +----------------------------------------------------------------------+
    | Copyright (c) The PHP Group                                          |
    +----------------------------------------------------------------------+
@@ -16,6 +14,10 @@
    +----------------------------------------------------------------------+
  */
 
+#ifdef __sun
+/* to enable 'new' ancillary data layout instead */
+# define _XPG4_2
+#endif
 #include <php.h>
 #include "php_sockets.h"
 #include "sendrecvmsg.h"
@@ -68,12 +70,11 @@ inline ssize_t sendmsg(int sockfd, const struct msghdr *msg, int flags)
 }
 #endif
 
-#define LONG_CHECK_VALID_INT(l) \
+#define LONG_CHECK_VALID_INT(l, arg_pos) \
 	do { \
-		if ((l) < INT_MIN && (l) > INT_MAX) { \
-			php_error_docref(NULL, E_WARNING, "The value " ZEND_LONG_FMT " does not fit inside " \
-					"the boundaries of a native integer", (l)); \
-			return; \
+		if ((l) < INT_MIN || (l) > INT_MAX) { \
+			zend_argument_value_error((arg_pos), "must be between %d and %d", INT_MIN, INT_MAX); \
+			RETURN_THROWS(); \
 		} \
 	} while (0)
 
@@ -124,8 +125,13 @@ static void init_ancillary_registry(void)
 #endif
 
 #ifdef SO_PASSCRED
+#ifdef ANC_CREDS_UCRED
 	PUT_ENTRY(sizeof(struct ucred), 0, 0, from_zval_write_ucred,
 			to_zval_read_ucred, SOL_SOCKET, SCM_CREDENTIALS);
+#else
+	PUT_ENTRY(sizeof(struct cmsgcred), 0, 0, from_zval_write_ucred,
+			to_zval_read_ucred, SOL_SOCKET, SCM_CREDS);
+#endif
 #endif
 
 #ifdef SCM_RIGHTS
@@ -175,16 +181,14 @@ PHP_FUNCTION(socket_sendmsg)
 	ssize_t			res;
 
 	/* zmsg should be passed by ref */
-	if (zend_parse_parameters(ZEND_NUM_ARGS(), "ra|l", &zsocket, &zmsg, &flags) == FAILURE) {
-		return;
+	if (zend_parse_parameters(ZEND_NUM_ARGS(), "Oa|l", &zsocket, socket_ce, &zmsg, &flags) == FAILURE) {
+		RETURN_THROWS();
 	}
 
-	LONG_CHECK_VALID_INT(flags);
+	LONG_CHECK_VALID_INT(flags, 3);
 
-	if ((php_sock = (php_socket *)zend_fetch_resource(Z_RES_P(zsocket),
-					php_sockets_le_socket_name, php_sockets_le_socket())) == NULL) {
-		RETURN_FALSE;
-	}
+	php_sock = Z_SOCKET_P(zsocket);
+	ENSURE_SOCKET_VALID(php_sock);
 
 	msghdr = from_zval_run_conversions(zmsg, php_sock, from_zval_write_msghdr_send,
 			sizeof(*msghdr), "msghdr", &allocations, &err);
@@ -197,14 +201,13 @@ PHP_FUNCTION(socket_sendmsg)
 	res = sendmsg(php_sock->bsd_socket, msghdr, (int)flags);
 
 	if (res != -1) {
-		zend_llist_destroy(allocations);
-		efree(allocations);
-
-		RETURN_LONG((zend_long)res);
+		RETVAL_LONG((zend_long)res);
 	} else {
-		PHP_SOCKET_ERROR(php_sock, "error in sendmsg", errno);
-		RETURN_FALSE;
+		PHP_SOCKET_ERROR(php_sock, "Error in sendmsg", errno);
+		RETVAL_FALSE;
 	}
+
+	allocations_dispose(&allocations);
 }
 
 PHP_FUNCTION(socket_recvmsg)
@@ -219,17 +222,14 @@ PHP_FUNCTION(socket_recvmsg)
 	struct err_s	err = {0};
 
 	//ssize_t recvmsg(int sockfd, struct msghdr *msg, int flags);
-	if (zend_parse_parameters(ZEND_NUM_ARGS(), "ra|l",
-			&zsocket, &zmsg, &flags) == FAILURE) {
-		return;
+	if (zend_parse_parameters(ZEND_NUM_ARGS(), "Oa|l", &zsocket, socket_ce, &zmsg, &flags) == FAILURE) {
+		RETURN_THROWS();
 	}
 
-	LONG_CHECK_VALID_INT(flags);
+	LONG_CHECK_VALID_INT(flags, 3);
 
-	if ((php_sock = (php_socket *)zend_fetch_resource(Z_RES_P(zsocket),
-					php_sockets_le_socket_name, php_sockets_le_socket())) == NULL) {
-		RETURN_FALSE;
-	}
+	php_sock = Z_SOCKET_P(zsocket);
+	ENSURE_SOCKET_VALID(php_sock);
 
 	msghdr = from_zval_run_conversions(zmsg, php_sock, from_zval_write_msghdr_recv,
 			sizeof(*msghdr), "msghdr", &allocations, &err);
@@ -254,7 +254,6 @@ PHP_FUNCTION(socket_recvmsg)
 
 		/* we don;t need msghdr anymore; free it */
 		msghdr = NULL;
-		allocations_dispose(&allocations);
 
 		zval_ptr_dtor(zmsg);
 		if (!err.has_error) {
@@ -265,14 +264,15 @@ PHP_FUNCTION(socket_recvmsg)
 			/* no need to destroy/free zres -- it's NULL in this circumstance */
 			assert(zres == NULL);
 		}
+		RETVAL_LONG((zend_long)res);
 	} else {
 		SOCKETS_G(last_error) = errno;
-		php_error_docref(NULL, E_WARNING, "error in recvmsg [%d]: %s",
+		php_error_docref(NULL, E_WARNING, "Error in recvmsg [%d]: %s",
 				errno, sockets_strerror(errno));
-		RETURN_FALSE;
+		RETVAL_FALSE;
 	}
 
-	RETURN_LONG((zend_long)res);
+	allocations_dispose(&allocations);
 }
 
 PHP_FUNCTION(socket_cmsg_space)
@@ -284,32 +284,37 @@ PHP_FUNCTION(socket_cmsg_space)
 
 	if (zend_parse_parameters(ZEND_NUM_ARGS(), "ll|l",
 			&level, &type, &n) == FAILURE) {
-		return;
+		RETURN_THROWS();
 	}
 
-	LONG_CHECK_VALID_INT(level);
-	LONG_CHECK_VALID_INT(type);
-	LONG_CHECK_VALID_INT(n);
+	LONG_CHECK_VALID_INT(level, 1);
+	LONG_CHECK_VALID_INT(type, 2);
+	LONG_CHECK_VALID_INT(n, 3);
 
 	if (n < 0) {
-		php_error_docref(NULL, E_WARNING, "The third argument "
-				"cannot be negative");
-		return;
+		zend_argument_value_error(3, "must be greater than or equal to 0");
+		RETURN_THROWS();
 	}
 
 	entry = get_ancillary_reg_entry(level, type);
 	if (entry == NULL) {
-		php_error_docref(NULL, E_WARNING, "The pair level " ZEND_LONG_FMT "/type " ZEND_LONG_FMT " is "
-				"not supported by PHP", level, type);
-		return;
+		zend_value_error("Pair level " ZEND_LONG_FMT " and/or type " ZEND_LONG_FMT " is not supported",
+			level, type);
+		RETURN_THROWS();
 	}
 
-	if (entry->var_el_size > 0 && n > (zend_long)((ZEND_LONG_MAX - entry->size -
-			CMSG_SPACE(0) - 15L) / entry->var_el_size)) {
-		/* the -15 is to account for any padding CMSG_SPACE may add after the data */
-		php_error_docref(NULL, E_WARNING, "The value for the "
-				"third argument (" ZEND_LONG_FMT ") is too large", n);
-		return;
+	if (entry->var_el_size > 0) {
+		/* Leading underscore to avoid symbol collision on AIX. */
+		size_t _rem_size = ZEND_LONG_MAX - entry->size;
+		size_t n_max = _rem_size / entry->var_el_size;
+		size_t size = entry->size + n * entry->var_el_size;
+		size_t total_size = CMSG_SPACE(size);
+		if (n > n_max /* zend_long overflow */
+			|| total_size > ZEND_LONG_MAX
+			|| total_size < size /* align overflow */) {
+			zend_argument_value_error(3, "is too large");
+			RETURN_THROWS();
+		}
 	}
 
 	RETURN_LONG((zend_long)CMSG_SPACE(entry->size + n * entry->var_el_size));
@@ -360,7 +365,7 @@ int php_do_setsockopt_ipv6_rfc3542(php_socket *php_sock, int level, int optname,
 dosockopt:
 	retval = setsockopt(php_sock->bsd_socket, level, optname, opt_ptr, optlen);
 	if (retval != 0) {
-		PHP_SOCKET_ERROR(php_sock, "unable to set socket option", errno);
+		PHP_SOCKET_ERROR(php_sock, "Unable to set socket option", errno);
 	}
 	allocations_dispose(&allocations);
 
@@ -440,7 +445,11 @@ void php_socket_sendrecvmsg_init(INIT_FUNC_ARGS)
 	REGISTER_LONG_CONSTANT("SCM_RIGHTS",			SCM_RIGHTS,			CONST_CS | CONST_PERSISTENT);
 #endif
 #ifdef SO_PASSCRED
+#ifdef SCM_CREDENTIALS
 	REGISTER_LONG_CONSTANT("SCM_CREDENTIALS",		SCM_CREDENTIALS,	CONST_CS | CONST_PERSISTENT);
+#else
+	REGISTER_LONG_CONSTANT("SCM_CREDS",		SCM_CREDS,	CONST_CS | CONST_PERSISTENT);
+#endif
 	REGISTER_LONG_CONSTANT("SO_PASSCRED",			SO_PASSCRED,		CONST_CS | CONST_PERSISTENT);
 #endif
 
