@@ -1,6 +1,4 @@
 /*
-   +----------------------------------------------------------------------+
-   | PHP Version 7                                                        |
    +----------------------------------------------------------------------+
    | Copyright (c) The PHP Group                                          |
    +----------------------------------------------------------------------+
@@ -53,9 +51,6 @@
 /* Used to check DES salts to ensure that they contain only valid characters */
 #define IS_VALID_SALT_CHARACTER(c) (((c) >= '.' && (c) <= '9') || ((c) >= 'A' && (c) <= 'Z') || ((c) >= 'a' && (c) <= 'z'))
 
-#define DES_INVALID_SALT_ERROR "Supplied salt is not valid for DES. Possible bug in provided salt format."
-
-
 PHP_MINIT_FUNCTION(crypt) /* {{{ */
 {
 	REGISTER_LONG_CONSTANT("CRYPT_SALT_LENGTH", PHP_MAX_SALT_LEN, CONST_CS | CONST_PERSISTENT);
@@ -84,21 +79,15 @@ PHP_MSHUTDOWN_FUNCTION(crypt) /* {{{ */
 }
 /* }}} */
 
-static unsigned char itoa64[] = "./0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz";
-
-static void php_to64(char *s, int n) /* {{{ */
-{
-	while (--n >= 0) {
-		*s = itoa64[*s & 0x3f];
-		s++;
-	}
-}
-/* }}} */
-
 PHPAPI zend_string *php_crypt(const char *password, const int pass_len, const char *salt, int salt_len, zend_bool quiet)
 {
 	char *crypt_res;
 	zend_string *result;
+
+	if (salt[0] == '*' && (salt[1] == '0' || salt[1] == '1')) {
+		return NULL;
+	}
+
 /* Windows (win32/crypt) has a stripped down version of libxcrypt and
 	a CryptoApi md5_crypt implementation */
 #if PHP_USE_PHP_CRYPT_R
@@ -146,6 +135,7 @@ PHPAPI zend_string *php_crypt(const char *password, const int pass_len, const ch
 		} else if (
 				salt[0] == '$' &&
 				salt[1] == '2' &&
+				salt[2] != 0 &&
 				salt[3] == '$') {
 			char output[PHP_MAX_SALT_LEN + 1];
 
@@ -160,22 +150,9 @@ PHPAPI zend_string *php_crypt(const char *password, const int pass_len, const ch
 				ZEND_SECURE_ZERO(output, PHP_MAX_SALT_LEN + 1);
 				return result;
 			}
-        } else if (salt[0] == '*' && (salt[1] == '0' || salt[1] == '1')) {
-            return NULL;
-		} else {
+		} else if (salt[0] == '_'
+				|| (IS_VALID_SALT_CHARACTER(salt[0]) && IS_VALID_SALT_CHARACTER(salt[1]))) {
 			/* DES Fallback */
-
-			/* Only check the salt if it's not EXT_DES */
-			if (salt[0] != '_') {
-				/* DES style hashes */
-				if (!IS_VALID_SALT_CHARACTER(salt[0]) || !IS_VALID_SALT_CHARACTER(salt[1])) {
-					if (!quiet) {
-						/* error consistently about invalid DES fallbacks */
-						php_error_docref(NULL, E_DEPRECATED, DES_INVALID_SALT_ERROR);
-					}
-				}
-			}
-
 			memset(&buffer, 0, sizeof(buffer));
 			_crypt_extended_init_r();
 
@@ -186,29 +163,23 @@ PHPAPI zend_string *php_crypt(const char *password, const int pass_len, const ch
 				result = zend_string_init(crypt_res, strlen(crypt_res), 0);
 				return result;
 			}
+		} else {
+			/* Unknown hash type */
+			return NULL;
 		}
 	}
 #else
 
-	if (salt[0] != '$' && salt[0] != '_' && (!IS_VALID_SALT_CHARACTER(salt[0]) || !IS_VALID_SALT_CHARACTER(salt[1]))) {
-		if (!quiet) {
-			/* error consistently about invalid DES fallbacks */
-			php_error_docref(NULL, E_DEPRECATED, DES_INVALID_SALT_ERROR);
-		}
-	}
-
 # if defined(HAVE_CRYPT_R) && (defined(_REENTRANT) || defined(_THREAD_SAFE))
-	{
 #  if defined(CRYPT_R_STRUCT_CRYPT_DATA)
-		struct crypt_data buffer;
-		memset(&buffer, 0, sizeof(buffer));
+	struct crypt_data buffer;
+	memset(&buffer, 0, sizeof(buffer));
 #  elif defined(CRYPT_R_CRYPTD)
-		CRYPTD buffer;
+	CRYPTD buffer;
 #  else
 #   error Data struct used by crypt_r() is unknown. Please report.
 #  endif
-		crypt_res = crypt_r(password, salt, &buffer);
-	}
+	crypt_res = crypt_r(password, salt, &buffer);
 # elif defined(HAVE_CRYPT)
 	crypt_res = crypt(password, salt);
 # else
@@ -226,8 +197,7 @@ PHPAPI zend_string *php_crypt(const char *password, const int pass_len, const ch
 /* }}} */
 
 
-/* {{{ proto string crypt(string str [, string salt])
-   Hash a string */
+/* {{{ Hash a string */
 PHP_FUNCTION(crypt)
 {
 	char salt[PHP_MAX_SALT_LEN + 1];
@@ -235,9 +205,8 @@ PHP_FUNCTION(crypt)
 	size_t str_len, salt_in_len = 0;
 	zend_string *result;
 
-	ZEND_PARSE_PARAMETERS_START(1, 2)
+	ZEND_PARSE_PARAMETERS_START(2, 2)
 		Z_PARAM_STRING(str, str_len)
-		Z_PARAM_OPTIONAL
 		Z_PARAM_STRING(salt_in, salt_in_len)
 	ZEND_PARSE_PARAMETERS_END();
 
@@ -246,23 +215,9 @@ PHP_FUNCTION(crypt)
 	/* This will produce suitable results if people depend on DES-encryption
 	 * available (passing always 2-character salt). At least for glibc6.1 */
 	memset(&salt[1], '$', PHP_MAX_SALT_LEN - 1);
+	memcpy(salt, salt_in, MIN(PHP_MAX_SALT_LEN, salt_in_len));
 
-	if (salt_in) {
-		memcpy(salt, salt_in, MIN(PHP_MAX_SALT_LEN, salt_in_len));
-	} else {
-		php_error_docref(NULL, E_NOTICE, "No salt parameter was specified. You must use a randomly generated salt and a strong hash function to produce a secure hash.");
-	}
-
-	/* The automatic salt generation covers standard DES, md5-crypt and Blowfish (simple) */
-	if (!*salt) {
-		memcpy(salt, "$1$", 3);
-		php_random_bytes_throw(&salt[3], 8);
-		php_to64(&salt[3], 8);
-		strncpy(&salt[11], "$", PHP_MAX_SALT_LEN - 11);
-		salt_in_len = strlen(salt);
-	} else {
-		salt_in_len = MIN(PHP_MAX_SALT_LEN, salt_in_len);
-	}
+	salt_in_len = MIN(PHP_MAX_SALT_LEN, salt_in_len);
 	salt[salt_in_len] = '\0';
 
 	if ((result = php_crypt(str, (int)str_len, salt, (int)salt_in_len, 0)) == NULL) {
