@@ -1,8 +1,6 @@
 /*
    +----------------------------------------------------------------------+
-   | PHP Version 7                                                        |
-   +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2018 The PHP Group                                |
+   | Copyright (c) The PHP Group                                          |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -18,8 +16,6 @@
    +----------------------------------------------------------------------+
 */
 
-/* $Id$ */
-
 #include "php.h"
 #include "md5.h"
 
@@ -42,13 +38,11 @@ PHPAPI void make_digest_ex(char *md5str, const unsigned char *digest, int len) /
 }
 /* }}} */
 
-/* {{{ proto string md5(string str, [ bool raw_output])
-   Calculate the md5 hash of a string */
-PHP_NAMED_FUNCTION(php_if_md5)
+/* {{{ Calculate the md5 hash of a string */
+PHP_FUNCTION(md5)
 {
 	zend_string *arg;
 	zend_bool raw_output = 0;
-	char md5str[33];
 	PHP_MD5_CTX context;
 	unsigned char digest[16];
 
@@ -58,32 +52,29 @@ PHP_NAMED_FUNCTION(php_if_md5)
 		Z_PARAM_BOOL(raw_output)
 	ZEND_PARSE_PARAMETERS_END();
 
-	md5str[0] = '\0';
 	PHP_MD5Init(&context);
 	PHP_MD5Update(&context, ZSTR_VAL(arg), ZSTR_LEN(arg));
 	PHP_MD5Final(digest, &context);
 	if (raw_output) {
 		RETURN_STRINGL((char *) digest, 16);
 	} else {
-		make_digest_ex(md5str, digest, 16);
-		RETVAL_STRING(md5str);
+		RETVAL_NEW_STR(zend_string_alloc(32, 0));
+		make_digest_ex(Z_STRVAL_P(return_value), digest, 16);
 	}
 
 }
 /* }}} */
 
-/* {{{ proto string md5_file(string filename [, bool raw_output])
-   Calculate the md5 hash of given filename */
-PHP_NAMED_FUNCTION(php_if_md5_file)
+/* {{{ Calculate the md5 hash of given filename */
+PHP_FUNCTION(md5_file)
 {
 	char          *arg;
 	size_t           arg_len;
 	zend_bool raw_output = 0;
-	char          md5str[33];
 	unsigned char buf[1024];
 	unsigned char digest[16];
 	PHP_MD5_CTX   context;
-	size_t           n;
+	ssize_t       n;
 	php_stream    *stream;
 
 	ZEND_PARSE_PARAMETERS_START(1, 2)
@@ -118,8 +109,8 @@ PHP_NAMED_FUNCTION(php_if_md5_file)
 	if (raw_output) {
 		RETURN_STRINGL((char *) digest, 16);
 	} else {
-		make_digest_ex(md5str, digest, 16);
-		RETVAL_STRING(md5str);
+		RETVAL_NEW_STR(zend_string_alloc(32, 0));
+		make_digest_ex(Z_STRVAL_P(return_value), digest, 16);
 	}
 }
 /* }}} */
@@ -172,8 +163,9 @@ PHP_NAMED_FUNCTION(php_if_md5_file)
  * doesn't work.
  */
 #if defined(__i386__) || defined(__x86_64__) || defined(__vax__)
+typedef ZEND_SET_ALIGNED(1, uint32_t unaligned_uint32_t);
 # define SET(n) \
-	(*(uint32_t *)&ptr[(n) * 4])
+	(*(unaligned_uint32_t *)&ptr[(n) * 4])
 # define GET(n) \
 	SET(n)
 #else
