@@ -1,6 +1,4 @@
 /*
-   +----------------------------------------------------------------------+
-   | PHP Version 7                                                        |
    +----------------------------------------------------------------------+
    | Copyright (c) The PHP Group                                          |
    +----------------------------------------------------------------------+
@@ -18,8 +16,7 @@
    +----------------------------------------------------------------------+
 */
 
-/* {{{ includes
-*/
+/* {{{ includes */
 #include <stdio.h>
 #include <stdlib.h>
 #include <errno.h>
@@ -76,13 +73,11 @@ static void php_object_property_dump(zend_property_info *prop_info, zval *zv, ze
 	}
 
 	if (Z_TYPE_P(zv) == IS_UNDEF) {
-		ZEND_ASSERT(prop_info->type);
-		php_printf("%*cuninitialized(%s%s)\n",
-			level + 1, ' ',
-			ZEND_TYPE_ALLOW_NULL(prop_info->type) ? "?" : "",
-			ZEND_TYPE_IS_CLASS(prop_info->type) ?
-				ZSTR_VAL(ZEND_TYPE_IS_CE(prop_info->type) ? ZEND_TYPE_CE(prop_info->type)->name : ZEND_TYPE_NAME(prop_info->type)) :
-				zend_get_type_by_const(ZEND_TYPE_CODE(prop_info->type)));
+		ZEND_ASSERT(ZEND_TYPE_IS_SET(prop_info->type));
+		zend_string *type_str = zend_type_to_string(prop_info->type);
+		php_printf("%*cuninitialized(%s)\n",
+			level + 1, ' ', ZSTR_VAL(type_str));
+		zend_string_release(type_str);
 	} else {
 		php_var_dump(zv, level + 2);
 	}
@@ -118,7 +113,7 @@ again:
 			php_printf("%sint(" ZEND_LONG_FMT ")\n", COMMON, Z_LVAL_P(struc));
 			break;
 		case IS_DOUBLE:
-			php_printf("%sfloat(%.*G)\n", COMMON, (int) EG(precision), Z_DVAL_P(struc));
+			php_printf_unchecked("%sfloat(%.*H)\n", COMMON, (int) PG(serialize_precision), Z_DVAL_P(struc));
 			break;
 		case IS_STRING:
 			php_printf("%sstring(%zd) \"", COMMON, Z_STRLEN_P(struc));
@@ -128,14 +123,12 @@ again:
 		case IS_ARRAY:
 			myht = Z_ARRVAL_P(struc);
 			if (!(GC_FLAGS(myht) & GC_IMMUTABLE)) {
-				if (level > 1) {
-					if (GC_IS_RECURSIVE(myht)) {
-						PUTS("*RECURSION*\n");
-						return;
-					}
-					GC_PROTECT_RECURSION(myht);
+				if (GC_IS_RECURSIVE(myht)) {
+					PUTS("*RECURSION*\n");
+					return;
 				}
 				GC_ADDREF(myht);
+				GC_PROTECT_RECURSION(myht);
 			}
 			count = zend_array_count(myht);
 			php_printf("%sarray(%d) {\n", COMMON, count);
@@ -143,9 +136,7 @@ again:
 				php_array_element_dump(val, num, key, level);
 			} ZEND_HASH_FOREACH_END();
 			if (!(GC_FLAGS(myht) & GC_IMMUTABLE)) {
-				if (level > 1) {
-					GC_UNPROTECT_RECURSION(myht);
-				}
+				GC_UNPROTECT_RECURSION(myht);
 				GC_DELREF(myht);
 			}
 			if (level > 1) {
@@ -212,8 +203,7 @@ again:
 }
 /* }}} */
 
-/* {{{ proto void var_dump(mixed var)
-   Dumps a string representation of variable to output */
+/* {{{ Dumps a string representation of variable to output */
 PHP_FUNCTION(var_dump)
 {
 	zval *args;
@@ -265,13 +255,10 @@ static void zval_object_property_dump(zend_property_info *prop_info, zval *zv, z
 		ZEND_PUTS("]=>\n");
 	}
 	if (prop_info && Z_TYPE_P(zv) == IS_UNDEF) {
-		ZEND_ASSERT(prop_info->type);
-		php_printf("%*cuninitialized(%s%s)\n",
-			level + 1, ' ',
-			ZEND_TYPE_ALLOW_NULL(prop_info->type) ? "?" : "",
-			ZEND_TYPE_IS_CLASS(prop_info->type) ?
-				ZSTR_VAL(ZEND_TYPE_IS_CE(prop_info->type) ? ZEND_TYPE_CE(prop_info->type)->name : ZEND_TYPE_NAME(prop_info->type)) :
-				zend_get_type_by_const(ZEND_TYPE_CODE(prop_info->type)));
+		zend_string *type_str = zend_type_to_string(prop_info->type);
+		php_printf("%*cuninitialized(%s)\n",
+			level + 1, ' ', ZSTR_VAL(type_str));
+		zend_string_release(type_str);
 	} else {
 		php_debug_zval_dump(zv, level + 2);
 	}
@@ -307,7 +294,7 @@ again:
 		php_printf("%sint(" ZEND_LONG_FMT ")\n", COMMON, Z_LVAL_P(struc));
 		break;
 	case IS_DOUBLE:
-		php_printf("%sfloat(%.*G)\n", COMMON, (int) EG(precision), Z_DVAL_P(struc));
+		php_printf_unchecked("%sfloat(%.*H)\n", COMMON, (int) PG(serialize_precision), Z_DVAL_P(struc));
 		break;
 	case IS_STRING:
 		php_printf("%sstring(%zd) \"", COMMON, Z_STRLEN_P(struc));
@@ -317,14 +304,12 @@ again:
 	case IS_ARRAY:
 		myht = Z_ARRVAL_P(struc);
 		if (!(GC_FLAGS(myht) & GC_IMMUTABLE)) {
-			if (level > 1) {
-				if (GC_IS_RECURSIVE(myht)) {
-					PUTS("*RECURSION*\n");
-					return;
-				}
-				GC_PROTECT_RECURSION(myht);
+			if (GC_IS_RECURSIVE(myht)) {
+				PUTS("*RECURSION*\n");
+				return;
 			}
 			GC_ADDREF(myht);
+			GC_PROTECT_RECURSION(myht);
 		}
 		count = zend_array_count(myht);
 		php_printf("%sarray(%d) refcount(%u){\n", COMMON, count, Z_REFCOUNTED_P(struc) ? Z_REFCOUNT_P(struc) - 1 : 1);
@@ -332,9 +317,7 @@ again:
 			zval_array_element_dump(val, index, key, level);
 		} ZEND_HASH_FOREACH_END();
 		if (!(GC_FLAGS(myht) & GC_IMMUTABLE)) {
-			if (level > 1) {
-				GC_UNPROTECT_RECURSION(myht);
-			}
+			GC_UNPROTECT_RECURSION(myht);
 			GC_DELREF(myht);
 		}
 		if (level > 1) {
@@ -397,8 +380,7 @@ again:
 }
 /* }}} */
 
-/* {{{ proto void debug_zval_dump(mixed var)
-   Dumps a string representation of an internal zend value to output. */
+/* {{{ Dumps a string representation of an internal zend value to output. */
 PHP_FUNCTION(debug_zval_dump)
 {
 	zval *args;
@@ -625,8 +607,7 @@ PHPAPI void php_var_export(zval *struc, int level) /* {{{ */
 }
 /* }}} */
 
-/* {{{ proto mixed var_export(mixed var [, bool return])
-   Outputs or returns a string representation of a variable */
+/* {{{ Outputs or returns a string representation of a variable */
 PHP_FUNCTION(var_export)
 {
 	zval *var;
@@ -749,8 +730,11 @@ static int php_var_serialize_call_sleep(zval *retval, zval *struc) /* {{{ */
 	}
 
 	if (!HASH_OF(retval)) {
+		zend_class_entry *ce;
+		ZEND_ASSERT(Z_TYPE_P(struc) == IS_OBJECT);
+		ce = Z_OBJCE_P(struc);
 		zval_ptr_dtor(retval);
-		php_error_docref(NULL, E_NOTICE, "__sleep should return an array only containing the names of instance-variables to serialize");
+		php_error_docref(NULL, E_WARNING, "%s::__sleep() should return an array only containing the names of instance-variables to serialize", ZSTR_VAL(ce->name));
 		return FAILURE;
 	}
 
@@ -760,16 +744,12 @@ static int php_var_serialize_call_sleep(zval *retval, zval *struc) /* {{{ */
 
 static int php_var_serialize_call_magic_serialize(zval *retval, zval *obj) /* {{{ */
 {
-	zval fname;
-	int res;
-
-	ZVAL_STRINGL(&fname, "__serialize", sizeof("__serialize") - 1);
 	BG(serialize_lock)++;
-	res = call_user_function(CG(function_table), obj, &fname, retval, 0, 0);
+	zend_call_known_instance_method_with_0_params(
+		Z_OBJCE_P(obj)->__serialize, Z_OBJ_P(obj), retval);
 	BG(serialize_lock)--;
-	zval_ptr_dtor_str(&fname);
 
-	if (res == FAILURE || Z_ISUNDEF_P(retval)) {
+	if (EG(exception)) {
 		zval_ptr_dtor(retval);
 		return FAILURE;
 	}
@@ -805,7 +785,7 @@ static int php_var_serialize_try_add_sleep_prop(
 
 	if (!zend_hash_add(ht, name, val)) {
 		php_error_docref(NULL, E_NOTICE,
-			"\"%s\" is returned from __sleep multiple times", ZSTR_VAL(error_name));
+			"\"%s\" is returned from __sleep() multiple times", ZSTR_VAL(error_name));
 		return SUCCESS;
 	}
 
@@ -825,13 +805,14 @@ static int php_var_serialize_get_sleep_props(
 	zend_hash_init(ht, zend_hash_num_elements(sleep_retval), NULL, ZVAL_PTR_DTOR, 0);
 	/* TODO: Rewrite this by fetching the property info instead of trying out different
 	 * name manglings? */
-	ZEND_HASH_FOREACH_VAL(sleep_retval, name_val) {
+	ZEND_HASH_FOREACH_VAL_IND(sleep_retval, name_val) {
 		zend_string *name, *tmp_name, *priv_name, *prot_name;
 
 		ZVAL_DEREF(name_val);
 		if (Z_TYPE_P(name_val) != IS_STRING) {
-			php_error_docref(NULL, E_NOTICE,
-					"__sleep should return an array only containing the names of instance-variables to serialize.");
+			php_error_docref(NULL, E_WARNING,
+					"%s::__sleep() should return an array only containing the names of instance-variables to serialize",
+					ZSTR_VAL(ce->name));
 		}
 
 		name = zval_get_tmp_string(name_val, &tmp_name);
@@ -877,9 +858,8 @@ static int php_var_serialize_get_sleep_props(
 			break;
 		}
 
-		php_error_docref(NULL, E_NOTICE,
+		php_error_docref(NULL, E_WARNING,
 			"\"%s\" returned as member variable from __sleep() but does not exist", ZSTR_VAL(name));
-		zend_hash_add(ht, name, &EG(uninitialized_zval));
 		zend_tmp_string_release(tmp_name);
 	} ZEND_HASH_FOREACH_END();
 
@@ -1012,14 +992,13 @@ again:
 				zend_bool incomplete_class;
 				uint32_t count;
 
-				if (zend_hash_str_exists(&ce->function_table, "__serialize", sizeof("__serialize")-1)) {
+				if (ce->__serialize) {
 					zval retval, obj;
 					zend_string *key;
 					zval *data;
 					zend_ulong index;
 
-					Z_ADDREF_P(struc);
-					ZVAL_OBJ(&obj, Z_OBJ_P(struc));
+					ZVAL_OBJ_COPY(&obj, Z_OBJ_P(struc));
 					if (php_var_serialize_call_magic_serialize(&retval, &obj) == FAILURE) {
 						if (!EG(exception)) {
 							smart_str_appendl(buf, "N;", 2);
@@ -1082,8 +1061,7 @@ again:
 				if (ce != PHP_IC_ENTRY && zend_hash_str_exists(&ce->function_table, "__sleep", sizeof("__sleep")-1)) {
 					zval retval, tmp;
 
-					Z_ADDREF_P(struc);
-					ZVAL_OBJ(&tmp, Z_OBJ_P(struc));
+					ZVAL_OBJ_COPY(&tmp, Z_OBJ_P(struc));
 
 					if (php_var_serialize_call_sleep(&retval, &tmp) == FAILURE) {
 						if (!EG(exception)) {
@@ -1136,7 +1114,7 @@ PHPAPI void php_var_serialize(smart_str *buf, zval *struc, php_serialize_data_t
 }
 /* }}} */
 
-PHPAPI php_serialize_data_t php_var_serialize_init() {
+PHPAPI php_serialize_data_t php_var_serialize_init(void) {
 	struct php_serialize_data *d;
 	/* fprintf(stderr, "SERIALIZE_INIT      == lock: %u, level: %u\n", BG(serialize_lock), BG(serialize).level); */
 	if (BG(serialize_lock) || !BG(serialize).level) {
@@ -1165,8 +1143,7 @@ PHPAPI void php_var_serialize_destroy(php_serialize_data_t d) {
 	}
 }
 
-/* {{{ proto string serialize(mixed variable)
-   Returns a string representation of variable (which can later be unserialized) */
+/* {{{ Returns a string representation of variable (which can later be unserialized) */
 PHP_FUNCTION(serialize)
 {
 	zval *struc;
@@ -1183,36 +1160,26 @@ PHP_FUNCTION(serialize)
 
 	if (EG(exception)) {
 		smart_str_free(&buf);
-		RETURN_FALSE;
+		RETURN_THROWS();
 	}
 
 	if (buf.s) {
 		RETURN_NEW_STR(buf.s);
 	} else {
-		RETURN_NULL();
+		RETURN_EMPTY_STRING();
 	}
 }
 /* }}} */
 
-/* {{{ proto mixed unserialize(string variable_representation[, array options])
-   Takes a string representation of variable and recreates it */
-PHP_FUNCTION(unserialize)
+/* {{{ Takes a string representation of variable and recreates it, subject to the optional unserialize options HashTable */
+PHPAPI void php_unserialize_with_options(zval *return_value, const char *buf, const size_t buf_len, HashTable *options, const char* function_name)
 {
-	char *buf = NULL;
-	size_t buf_len;
 	const unsigned char *p;
 	php_unserialize_data_t var_hash;
-	zval *options = NULL;
 	zval *retval;
 	HashTable *class_hash = NULL, *prev_class_hash;
 	zend_long prev_max_depth, prev_cur_depth;
 
-	ZEND_PARSE_PARAMETERS_START(1, 2)
-		Z_PARAM_STRING(buf, buf_len)
-		Z_PARAM_OPTIONAL
-		Z_PARAM_ARRAY(options)
-	ZEND_PARSE_PARAMETERS_END_EX(RETURN_FALSE);
-
 	if (buf_len == 0) {
 		RETURN_FALSE;
 	}
@@ -1226,10 +1193,9 @@ PHP_FUNCTION(unserialize)
 	if (options != NULL) {
 		zval *classes, *max_depth;
 
-		classes = zend_hash_str_find_deref(Z_ARRVAL_P(options), "allowed_classes", sizeof("allowed_classes")-1);
+		classes = zend_hash_str_find_deref(options, "allowed_classes", sizeof("allowed_classes")-1);
 		if (classes && Z_TYPE_P(classes) != IS_ARRAY && Z_TYPE_P(classes) != IS_TRUE && Z_TYPE_P(classes) != IS_FALSE) {
-			php_error_docref(NULL, E_WARNING, "allowed_classes option should be array or boolean");
-			RETVAL_FALSE;
+			zend_type_error("%s(): Option \"allowed_classes\" must be of type array|bool, %s given", function_name, zend_zval_type_name(classes));
 			goto cleanup;
 		}
 
@@ -1255,16 +1221,14 @@ PHP_FUNCTION(unserialize)
 		}
 		php_var_unserialize_set_allowed_classes(var_hash, class_hash);
 
-		max_depth = zend_hash_str_find_deref(Z_ARRVAL_P(options), "max_depth", sizeof("max_depth") - 1);
+		max_depth = zend_hash_str_find_deref(options, "max_depth", sizeof("max_depth") - 1);
 		if (max_depth) {
 			if (Z_TYPE_P(max_depth) != IS_LONG) {
-				php_error_docref(NULL, E_WARNING, "max_depth should be int");
-				RETVAL_FALSE;
+				zend_type_error("%s(): Option \"max_depth\" must be of type int, %s given", function_name, zend_zval_type_name(max_depth));
 				goto cleanup;
 			}
 			if (Z_LVAL_P(max_depth) < 0) {
-				php_error_docref(NULL, E_WARNING, "max_depth cannot be negative");
-				RETVAL_FALSE;
+				zend_value_error("%s(): Option \"max_depth\" must be greater than or equal to 0", function_name);
 				goto cleanup;
 			}
 
@@ -1317,29 +1281,44 @@ cleanup:
 }
 /* }}} */
 
-/* {{{ proto int memory_get_usage([bool real_usage])
-   Returns the allocated by PHP memory */
+/* {{{ Takes a string representation of variable and recreates it */
+PHP_FUNCTION(unserialize)
+{
+	char *buf = NULL;
+	size_t buf_len;
+	HashTable *options = NULL;
+
+	ZEND_PARSE_PARAMETERS_START(1, 2)
+		Z_PARAM_STRING(buf, buf_len)
+		Z_PARAM_OPTIONAL
+		Z_PARAM_ARRAY_HT(options)
+	ZEND_PARSE_PARAMETERS_END();
+
+	php_unserialize_with_options(return_value, buf, buf_len, options, "unserialize");
+}
+/* }}} */
+
+/* {{{ Returns the allocated by PHP memory */
 PHP_FUNCTION(memory_get_usage) {
 	zend_bool real_usage = 0;
 
 	ZEND_PARSE_PARAMETERS_START(0, 1)
 		Z_PARAM_OPTIONAL
 		Z_PARAM_BOOL(real_usage)
-	ZEND_PARSE_PARAMETERS_END_EX(RETURN_FALSE);
+	ZEND_PARSE_PARAMETERS_END();
 
 	RETURN_LONG(zend_memory_usage(real_usage));
 }
 /* }}} */
 
-/* {{{ proto int memory_get_peak_usage([bool real_usage])
-   Returns the peak allocated by PHP memory */
+/* {{{ Returns the peak allocated by PHP memory */
 PHP_FUNCTION(memory_get_peak_usage) {
 	zend_bool real_usage = 0;
 
 	ZEND_PARSE_PARAMETERS_START(0, 1)
 		Z_PARAM_OPTIONAL
 		Z_PARAM_BOOL(real_usage)
-	ZEND_PARSE_PARAMETERS_END_EX(RETURN_FALSE);
+	ZEND_PARSE_PARAMETERS_END();
 
 	RETURN_LONG(zend_memory_peak_usage(real_usage));
 }
