@@ -1,6 +1,4 @@
 /*
-  +----------------------------------------------------------------------+
-  | PHP Version 7                                                        |
   +----------------------------------------------------------------------+
   | Copyright (c) The PHP Group                                          |
   +----------------------------------------------------------------------+
@@ -25,16 +23,6 @@ extern zend_module_entry tidy_module_entry;
 #include "php_version.h"
 #define PHP_TIDY_VERSION PHP_VERSION
 
-#define TIDY_METHOD_MAP(name, func_name, arg_types) \
-	ZEND_NAMED_FE(name, ZEND_FN(func_name), arg_types)
-#define TIDY_NODE_METHOD(name)    PHP_FUNCTION(tnm_ ##name)
-#define TIDY_NODE_ME(name, param) TIDY_METHOD_MAP(name, tnm_ ##name, param)
-#define TIDY_NODE_PRIVATE_ME(name, param) ZEND_NAMED_ME(name, ZEND_FN(tnm_ ##name), param, ZEND_ACC_PRIVATE)
-#define TIDY_DOC_METHOD(name)     PHP_FUNCTION(tdm_ ##name)
-#define TIDY_DOC_ME(name, param)  TIDY_METHOD_MAP(name, tdm_ ##name, param)
-#define TIDY_ATTR_METHOD(name)    PHP_FUNCTION(tam_ ##name)
-#define TIDY_ATTR_ME(name, param) TIDY_METHOD_MAP(name, tam_ ##name, param)
-
 ZEND_BEGIN_MODULE_GLOBALS(tidy)
 	char *default_config;
 	zend_bool clean_output;
