@@ -66,6 +66,12 @@ function zend_weakmap_attach(object $object, mixed $value): bool {}
 function zend_weakmap_remove(object $object): bool {}
 function zend_weakmap_dump(): array {}
 
+function zend_get_current_func_name(): string {}
+
+#if defined(HAVE_LIBXML) && !defined(PHP_WIN32)
+function zend_test_override_libxml_global_state(): void {}
+#endif
+
 }
 
 namespace ZendTestNS {
