@@ -1,8 +1,6 @@
 /*
   +----------------------------------------------------------------------+
-  | PHP Version 7                                                        |
-  +----------------------------------------------------------------------+
-  | Copyright (c) 1997-2018 The PHP Group                                |
+  | Copyright (c) The PHP Group                                          |
   +----------------------------------------------------------------------+
   | This source file is subject to version 3.01 of the PHP license,      |
   | that is bundled with this package in the file LICENSE, and is        |
@@ -19,23 +17,7 @@
 #ifndef PHP_PS_TITLE_HEADER
 #define PHP_PS_TITLE_HEADER
 
-ZEND_BEGIN_ARG_INFO(arginfo_cli_set_process_title, 0)
-    ZEND_ARG_INFO(0, title)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO(arginfo_cli_get_process_title, 0)
-ZEND_END_ARG_INFO()
-
 PHP_FUNCTION(cli_set_process_title);
 PHP_FUNCTION(cli_get_process_title);
 
 #endif
-
-/*
- * Local variables:
- * tab-width: 4
- * c-basic-offset: 4
- * End:
- * vim600: noet sw=4 ts=4 fdm=marker
- * vim<600: noet sw=4 ts=4
- */
