@@ -1,8 +1,6 @@
 /*
    +----------------------------------------------------------------------+
-   | PHP Version 5                                                        |
-   +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2015 The PHP Group                                |
+   | Copyright (c) The PHP Group                                          |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -16,14 +14,12 @@
    +----------------------------------------------------------------------+
 */
 
-/* $Id$ */
-
 #include "../fpm_config.h"
 #include "../fpm_events.h"
 #include "../fpm.h"
 #include "../zlog.h"
 
-#if HAVE_DEVPOLL
+#ifdef HAVE_DEVPOLL
 
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -33,7 +29,7 @@
 #include <errno.h>
 
 static int fpm_event_devpoll_init(int max);
-static int fpm_event_devpoll_clean();
+static int fpm_event_devpoll_clean(void);
 static int fpm_event_devpoll_wait(struct fpm_event_queue_s *queue, unsigned long int timeout);
 static int fpm_event_devpoll_add(struct fpm_event_s *ev);
 static int fpm_event_devpoll_remove(struct fpm_event_s *ev);
@@ -45,7 +41,7 @@ static struct fpm_event_module_s devpoll_module = {
 	.clean = fpm_event_devpoll_clean,
 	.wait = fpm_event_devpoll_wait,
 	.add = fpm_event_devpoll_add,
-	.remove = fpm_event_devpoll_remove, 
+	.remove = fpm_event_devpoll_remove,
 };
 
 int dpfd = -1;
@@ -55,17 +51,16 @@ static int npollfds = 0;
 
 #endif /* HAVE_DEVPOLL */
 
-struct fpm_event_module_s *fpm_event_devpoll_module() /* {{{ */
+struct fpm_event_module_s *fpm_event_devpoll_module(void)
 {
-#if HAVE_DEVPOLL
+#ifdef HAVE_DEVPOLL
 	return &devpoll_module;
 #else
 	return NULL;
 #endif /* HAVE_DEVPOLL */
 }
-/* }}} */
 
-#if HAVE_DEVPOLL
+#ifdef HAVE_DEVPOLL
 
 /*
  * Init module
@@ -76,7 +71,7 @@ static int fpm_event_devpoll_init(int max) /* {{{ */
 
 	/* open /dev/poll for future usages */
 	dpfd = open("/dev/poll", O_RDWR);
-	if (dpfd < 0) {  
+	if (dpfd < 0) {
 		zlog(ZLOG_ERROR, "Unable to open /dev/poll");
 		return -1;
 	}
@@ -117,10 +112,10 @@ static int fpm_event_devpoll_init(int max) /* {{{ */
 /*
  * Clean the module
  */
-static int fpm_event_devpoll_clean() /* {{{ */
+static int fpm_event_devpoll_clean(void)
 {
 	/* close /dev/poll if open */
-	if (dpfd > -1) {  
+	if (dpfd > -1) {
 		close(dpfd);
 		dpfd = -1;
 	}
@@ -140,7 +135,6 @@ static int fpm_event_devpoll_clean() /* {{{ */
 	npollfds = 0;
 	return 0;
 }
-/* }}} */
 
 /*
  * wait for events or timeout
@@ -168,7 +162,7 @@ static int fpm_event_devpoll_wait(struct fpm_event_queue_s *queue, unsigned long
 		}
 	}
 
-	/* iterate throught triggered events */
+	/* iterate through triggered events */
 	for (i = 0; i < ret; i++) {
 
 		/* find the corresponding event */
@@ -176,7 +170,7 @@ static int fpm_event_devpoll_wait(struct fpm_event_queue_s *queue, unsigned long
 		while (q) {
 
 			/* found */
-			if (q->ev && q->ev->fd == active_pollfds[i].fd) {  
+			if (q->ev && q->ev->fd == active_pollfds[i].fd) {
 
 					/* fire the event */
 					fpm_event_fire(q->ev);
@@ -202,7 +196,7 @@ static int fpm_event_devpoll_add(struct fpm_event_s *ev) /* {{{ */
 {
 	struct pollfd pollfd;
 
-	/* fill pollfd with event informations */
+	/* fill pollfd with event information */
 	pollfd.fd = ev->fd;
 	pollfd.events = POLLIN;
 	pollfd.revents = 0;
@@ -227,7 +221,7 @@ static int fpm_event_devpoll_remove(struct fpm_event_s *ev) /* {{{ */
 {
 	struct pollfd pollfd;
 
-	/* fill pollfd with the same informations as fpm_event_devpoll_add */
+	/* fill pollfd with the same information as fpm_event_devpoll_add */
 	pollfd.fd = ev->fd;
 	pollfd.events = POLLIN | POLLREMOVE;
 	pollfd.revents = 0;
