@@ -1,8 +1,6 @@
 /*
    +----------------------------------------------------------------------+
-   | PHP Version 7                                                        |
-   +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2018 The PHP Group                                |
+   | Copyright (c) The PHP Group                                          |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -22,6 +20,6 @@
 #include "../fpm_config.h"
 #include "../fpm_events.h"
 
-struct fpm_event_module_s *fpm_event_epoll_module();
+struct fpm_event_module_s *fpm_event_epoll_module(void);
 
 #endif /* FPM_EVENTS_EPOLL_H */
