@@ -1,5 +1,3 @@
-
-	/* $Id: fpm_events.h,v 1.9 2008/05/24 17:38:47 anight Exp $ */
 	/* (c) 2007,2008 Andrei Nigmatulin */
 
 #ifndef FPM_EVENTS_H
@@ -8,6 +6,7 @@
 #define FPM_EV_TIMEOUT  (1 << 0)
 #define FPM_EV_READ     (1 << 1)
 #define FPM_EV_PERSIST  (1 << 2)
+#define FPM_EV_EDGE     (1 << 3)
 
 #define fpm_event_set_timer(ev, flags, cb, arg) fpm_event_set((ev), -1, (flags), (cb), (arg))
 
@@ -22,11 +21,30 @@ struct fpm_event_s {
 	short which;              /* type of event */
 };
 
-void fpm_event_loop();
+typedef struct fpm_event_queue_s {
+	struct fpm_event_queue_s *prev;
+	struct fpm_event_queue_s *next;
+	struct fpm_event_s *ev;
+} fpm_event_queue;
+
+struct fpm_event_module_s {
+	const char *name;
+	int support_edge_trigger;
+	int (*init)(int max_fd);
+	int (*clean)(void);
+	int (*wait)(struct fpm_event_queue_s *queue, unsigned long int timeout);
+	int (*add)(struct fpm_event_s *ev);
+	int (*remove)(struct fpm_event_s *ev);
+};
+
+void fpm_event_loop(int err);
 void fpm_event_fire(struct fpm_event_s *ev);
-int fpm_event_init_main();
+int fpm_event_init_main(void);
 int fpm_event_set(struct fpm_event_s *ev, int fd, int flags, void (*callback)(struct fpm_event_s *, short, void *), void *arg);
 int fpm_event_add(struct fpm_event_s *ev, unsigned long int timeout);
 int fpm_event_del(struct fpm_event_s *ev);
+int fpm_event_pre_init(char *machanism);
+const char *fpm_event_machanism_name(void);
+int fpm_event_support_edge_trigger(void);
 
 #endif
