@@ -1,5 +1,3 @@
-
-	/* $Id: fpm_process_ctl.h,v 1.6 2008/07/20 21:33:10 anight Exp $ */
 	/* (c) 2007,2008 Andrei Nigmatulin */
 
 #ifndef FPM_PROCESS_CTL_H
@@ -18,14 +16,14 @@
 struct fpm_child_s;
 
 void fpm_pctl(int new_state, int action);
-int fpm_pctl_can_spawn_children();
+int fpm_pctl_can_spawn_children(void);
 int fpm_pctl_kill(pid_t pid, int how);
 void fpm_pctl_kill_all(int signo);
 void fpm_pctl_heartbeat(struct fpm_event_s *ev, short which, void *arg);
 void fpm_pctl_perform_idle_server_maintenance_heartbeat(struct fpm_event_s *ev, short which, void *arg);
 void fpm_pctl_on_socket_accept(struct fpm_event_s *ev, short which, void *arg);
-int fpm_pctl_child_exited();
-int fpm_pctl_init_main();
+int fpm_pctl_child_exited(void);
+int fpm_pctl_init_main(void);
 
 
 enum {
@@ -46,8 +44,8 @@ enum {
 	FPM_PCTL_TERM,
 	FPM_PCTL_STOP,
 	FPM_PCTL_CONT,
-	FPM_PCTL_QUIT
+	FPM_PCTL_QUIT,
+	FPM_PCTL_KILL
 };
 
 #endif
-
